@@ -1,13 +1,11 @@
 """Main Layout: Layout for the Main page in the Artifact Viewer"""
+
 from dash import html, dcc, dash_table
 import dash_bootstrap_components as dbc
 
 # Get the SageWorks Version and License ID
 import sageworks
-from sageworks.utils.license_manager import LicenseManager
-
-sageworks_version = sageworks.__version__.split("+")[0].strip()
-sageworks_license = LicenseManager.get_license_id()
+from sageworks.utils.config_manager import ConfigManager
 
 
 def main_layout(
@@ -18,6 +16,11 @@ def main_layout(
     models: dash_table.DataTable,
     endpoints: dash_table.DataTable,
 ) -> html.Div:
+    """Main Layout for the Dashboard"""
+    sageworks_version = sageworks.__version__.split("+")[0].strip()
+    cm = ConfigManager()
+    license_id = cm.get_license_id()
+
     # Just put all the tables in as Rows for Now (do something fancy later)
     layout = html.Div(
         children=[
@@ -30,12 +33,12 @@ def main_layout(
                             html.Span(
                                 f" v {sageworks_version}",
                                 style={
-                                    "color": "rgb(200, 140, 200)",
+                                    "color": "rgb(180, 120, 180)",
                                     "fontSize": 15,
                                 },
                             ),
                             html.Span(
-                                f" ({sageworks_license})",
+                                f" ({license_id})",
                                 style={
                                     "color": "rgb(140, 140, 200)",
                                     "fontSize": 15,
