@@ -1,11 +1,16 @@
 """Callbacks for the Model Subpage Web User Interface"""
+
 from dash import Dash, no_update
 from dash.dependencies import Input, Output, State
 
 # SageWorks Imports
 from sageworks.views.model_web_view import ModelWebView
-from sageworks.web_components import table, model_markdown, model_metrics
+from sageworks.web_components import (
+    table,
+    model_plot,
+)
 from sageworks.utils.pandas_utils import deserialize_aws_broker_data
+from sageworks.api.model import Model
 
 
 def update_models_table(app: Dash):
@@ -47,19 +52,15 @@ def table_row_select(app: Dash, table_name: str):
         return row_style
 
 
-# Updates the model details when a model row is selected
-def update_model_detail_components(app: Dash, model_web_view: ModelWebView):
+# Updates the model plot when a model row is selected
+def update_model_plot_component(app: Dash):
     @app.callback(
-        [
-            Output("model_details_header", "children"),
-            Output("model_details", "children"),
-            Output("model_metrics", "figure"),
-        ],
-        Input("models_table", "derived_viewport_selected_row_ids"),
-        State("models_table", "data"),
+        Output("model_plot", "figure"),
+        Input("model_details-dropdown", "value"),
+        [State("models_table", "data"), State("models_table", "derived_viewport_selected_row_ids")],
         prevent_initial_call=True,
     )
-    def generate_model_details_figures(selected_rows, table_data):
+    def generate_model_plot_figure(inference_run, table_data, selected_rows):
         # Check for no selected rows
         if not selected_rows or selected_rows[0] is None:
             return no_update
@@ -67,20 +68,13 @@ def update_model_detail_components(app: Dash, model_web_view: ModelWebView):
         # Get the selected row data and grab the uuid
         selected_row_data = table_data[selected_rows[0]]
         model_uuid = selected_row_data["uuid"]
-        print(f"Model UUID: {model_uuid}")
+        m = Model(model_uuid, legacy=True)
 
-        # Set the Header Text
-        header = f"Details: {model_uuid}"
-
-        # Model Details
-        model_details = model_web_view.model_details(model_uuid)
-        model_details_markdown = model_markdown.ModelMarkdown().generate_markdown(model_details)
-
-        # Model Metrics
-        model_metrics_figure = model_metrics.ModelMetrics().generate_component_figure(model_details)
+        # Model Details Markdown component
+        model_plot_fig = model_plot.ModelPlot().update_contents(m, inference_run)
 
         # Return the details/markdown for these data details
-        return [header, model_details_markdown, model_metrics_figure]
+        return model_plot_fig
 
 
 # Updates the plugin component when a model row is selected
@@ -100,6 +94,6 @@ def update_plugin(app: Dash, plugin, model_web_view: ModelWebView):
         selected_row_data = table_data[selected_rows[0]]
         model_uuid = selected_row_data["uuid"]
 
-        # Get the model details and send it to the plugin
-        model_details = model_web_view.model_details(model_uuid)
-        return plugin.generate_component_figure(model_details)
+        # Instantiate the Model and send it to the plugin
+        model = Model(model_uuid, legacy=True)
+        return plugin.update_contents(model)
