@@ -1,4 +1,5 @@
 """Layout for the Models page"""
+
 from typing import Any
 from dash import dcc, html, dash_table
 import dash_bootstrap_components as dbc
@@ -6,8 +7,8 @@ import dash_bootstrap_components as dbc
 
 def models_layout(
     models_table: dash_table.DataTable,
-    model_details: dcc.Markdown,
-    model_metrics: dcc.Graph,
+    model_details: html.Div,
+    model_plot: dcc.Graph,
     **kwargs: Any,
 ) -> html.Div:
     # Generate rows for each plugin
@@ -32,28 +33,16 @@ def models_layout(
             dbc.Row(
                 [
                     # Column 1: Model Details
+                    dbc.Col(model_details, width=4),
+                    # Column 2: Model Plot and Plugins
                     dbc.Col(
                         [
                             dbc.Row(
-                                html.H3("Details", id="model_details_header"),
+                                html.H3("Model Plot", id="model_plot_header"),
                                 style={"padding": "30px 0px 10px 0px"},
                             ),
                             dbc.Row(
-                                model_details,
-                                style={"padding": "0px 0px 30px 0px"},
-                            ),
-                        ],
-                        width=4,
-                    ),
-                    # Column 2: Model Metrics and Plugins
-                    dbc.Col(
-                        [
-                            dbc.Row(
-                                html.H3("Model Testing", id="model_metrics_header"),
-                                style={"padding": "30px 0px 10px 0px"},
-                            ),
-                            dbc.Row(
-                                model_metrics,
+                                model_plot,
                                 style={"padding": "30px 0px 10px 0px"},
                             ),
                             dbc.Row(
