@@ -1,5 +1,5 @@
 """Models:  A SageWorks Web Interface to view, and interact with Models"""
-import os
+
 from dash import register_page
 import dash
 from dash_bootstrap_templates import load_figure_template
@@ -9,9 +9,14 @@ from .layout import models_layout
 from . import callbacks
 
 # SageWorks Imports
-from sageworks.web_components import table, model_markdown, model_metrics, plugin_loader
-from sageworks.web_components.plugin_interface import PluginType
+from sageworks.web_components import (
+    table,
+    model_details,
+    model_plot,
+)
+from sageworks.web_components.plugin_interface import PluginPage
 from sageworks.views.model_web_view import ModelWebView
+from sageworks.utils.plugin_manager import PluginManager
 
 # Register this page with Dash
 register_page(
@@ -32,20 +37,22 @@ models_table = table.Table().create_component(
 )
 
 # Create a Markdown component to display the model details
-model_details = model_markdown.ModelMarkdown().create_component("model_details")
+model_details = model_details.ModelDetails()
+model_details_component = model_details.create_component("model_details")
 
-# Create a Model Metrics component to display the model metrics
-model_metrics = model_metrics.ModelMetrics().create_component("model_metrics")
+# Create a Model Plot component to display the model metrics
+model_plot_component = model_plot.ModelPlot().create_component("model_plot")
 
 # Capture our components in a dictionary to send off to the layout
 components = {
     "models_table": models_table,
-    "model_details": model_details,
-    "model_metrics": model_metrics,
+    "model_details": model_details_component,
+    "model_plot": model_plot_component,
 }
 
-# Load the plugins from the sageworks_plugins directory
-plugins = plugin_loader.load_plugins_from_dir(os.getenv("SAGEWORKS_PLUGINS"), PluginType.MODEL)
+# Load any web components plugins of type 'model'
+pm = PluginManager()
+plugins = pm.get_list_of_web_plugins(plugin_page=PluginPage.MODEL)
 
 # Add the plugins to the components dictionary
 for plugin in plugins:
@@ -58,10 +65,11 @@ layout = models_layout(**components)
 # Setup our callbacks/connections
 app = dash.get_app()
 callbacks.update_models_table(app)
+model_details.register_callbacks("models_table")
 
 # Callback for the model table
 callbacks.table_row_select(app, "models_table")
-callbacks.update_model_detail_components(app, model_broker)
+callbacks.update_model_plot_component(app)
 
 # For each plugin, set up a callback to update the plugin figure
 for plugin in plugins:
