@@ -1,4 +1,5 @@
 """Callbacks for the FeatureSets Subpage Web User Interface"""
+
 from datetime import datetime
 import dash
 from dash import Dash, html
@@ -195,7 +196,7 @@ def update_violin_plots(app: Dash, data_source_web_view: DataSourceWebView):
         if not selected_rows or selected_rows[0] is None:
             return dash.no_update
         smart_sample_rows = data_source_web_view.data_source_smart_sample(selected_rows[0])
-        return violin_plots.ViolinPlots().generate_component_figure(
+        return violin_plots.ViolinPlots().update_contents(
             smart_sample_rows,
             figure_args={
                 "box_visible": True,
