@@ -1,10 +1,11 @@
 """AWSAccountCheck runs a bunch of tests/checks to ensure SageWorks AWS Setup"""
-import os
+
 import sys
 import logging
 
 # SageWorks Imports
 from sageworks.aws_service_broker.aws_account_clamp import AWSAccountClamp
+from sageworks.utils.config_manager import ConfigManager, FatalConfigError
 
 
 class AWSAccountCheck:
@@ -18,10 +19,13 @@ class AWSAccountCheck:
         self.aws_clamp = AWSAccountClamp()
 
         # Grab our SageWorks Bucket
-        self.sageworks_bucket = os.environ.get("SAGEWORKS_BUCKET")
-        if self.sageworks_bucket is None:
-            print("Could not find ENV var for SAGEWORKS_BUCKET!")
-            sys.exit(1)
+        cm = ConfigManager()
+        if not cm.config_okay():
+            self.log.error("SageWorks Configuration Incomplete...")
+            self.log.error("Run the 'sageworks' command and follow the prompts...")
+            raise FatalConfigError()
+
+        self.sageworks_bucket = cm.get_config("SAGEWORKS_BUCKET")
 
     def check_s3_bucket_subfolders(self):
         """Check if the SageWorks S3 Bucket is set up and has the correct sub-folders"""
