@@ -1,4 +1,5 @@
 """FeatureSpider: A Spider for data/feature investigation and QA"""
+
 import numpy as np
 import pandas as pd
 from sklearn.neighbors import KNeighborsRegressor
@@ -8,33 +9,51 @@ from sklearn.preprocessing import StandardScaler
 
 # Feature Spider Class
 class FeatureSpider:
-    def __init__(self, df: pd.DataFrame, features: list, id_column: str, target_column: str):
+    def __init__(self, df: pd.DataFrame, features: list, id_column: str, target_column: str, neighbors: int = 5):
         """FeatureSpider: A Spider for data/feature investigation and QA
+
         Args:
              df: Pandas DataFrame
              features: List of feature column names
              id_column: Name of the ID column
              target_column: Name of the target column
+             neighbors: Number of neighbors to use in the KNN model (default: 5)
         """
-        # Check for expected columns (used later)
+        # Check for expected columns
         for column in [id_column, target_column] + features:
             if column not in df.columns:
                 print(f"DataFrame does not have required {column} Column!")
                 return
 
         # Set internal vars that are used later
-        self.df = df
+        self.df = df.copy()
         self.id_column = id_column
         self.target_column = target_column
         self.features = features
 
+        # Check for NaNs in the features and log the percentage
+        for feature in features:
+            nan_count = df[feature].isna().sum()
+            if nan_count > 0:
+                print(f"Feature '{feature}' has {nan_count} NaNs ({nan_count / len(df) * 100:.2f}%).")
+
+        # Remove and NaNs or INFs in the features
+        print(f"Dataframe Shape before NaN/INF removal {self.df.shape}")
+        self.df[features] = self.df[features].replace([float("inf"), float("-inf")], pd.NA)
+        self.df = self.df.dropna(subset=features).reset_index(drop=True)
+        print(f"Dataframe Shape after NaN/INF removal {self.df.shape}")
+
+        # Impute NaNs with the mean value for each feature
+        # imputer = SimpleImputer(strategy="mean")
+        # df[features] = imputer.fit_transform(df[features])
+
         # Build our KNN model pipeline with StandardScalar
-        knn = KNeighborsRegressor(n_neighbors=10, weights="distance")
+        knn = KNeighborsRegressor(n_neighbors=neighbors, weights="distance")
         self.pipe = make_pipeline(StandardScaler(), knn)
 
         # Fit Model on features and target
-        y = df[self.target_column]
-        X = df[self.features]
+        y = self.df[self.target_column]
+        X = self.df[self.features]
         self.pipe.fit(X, y)
 
         # Grab the Standard Scalar and KNN from the pipeline model
@@ -42,6 +61,9 @@ class FeatureSpider:
         self.scalar = self.pipe["standardscaler"]
         self.knn = self.pipe["kneighborsregressor"]
 
+        # This is for collection of the neighbor distances
+        self.neigh_distances = []
+
     def get_feature_matrix(self):
         """Return the KNN Model Internal Feature Matrix"""
         return self.knn._fit_X
@@ -109,6 +131,9 @@ class FeatureSpider:
         neigh_dist, neigh_ind = self.knn.kneighbors(x_scaled)
         target_values = self.knn._y[neigh_ind]
 
+        # Collect the neighbor distances by unpacking the list of lists
+        self.neigh_distances = [dist for sublist in neigh_dist for dist in sublist]
+
         # Note: We're assuming that the Neighbor Index is the same order/cardinality as the dataframe
         results_df["knn_target_values"] = [", ".join([str(val) for val in values]) for values in target_values]
         results_df["knn_distances"] = [", ".join([str(dis) for dis in distances]) for distances in neigh_dist]
@@ -247,6 +272,29 @@ def test():
     query_df = data_df[data_df["ID"] == "id_0"].copy()
     print(f_spider.neighbor_info(query_df))
 
+    # Feature matrix
+    print(f_spider.get_feature_matrix())
+
+
+def integration_test():
+    """Integration Test for the FeatureResolution Class"""
+    from sageworks.api.feature_set import FeatureSet
+    from sageworks.api.model import Model
+
+    # Grab a test dataframe
+    fs = FeatureSet("aqsol_mol_descriptors")
+    test_df = fs.pull_dataframe()
+
+    # Get the target and feature columns
+    m = Model("aqsol-mol-regression")
+    target_column = m.target()
+    feature_columns = m.features()
+
+    # Create the class and run the report
+    feature_spider = FeatureSpider(test_df, features=feature_columns, target_column=target_column, id_column="id")
+    feature_spider.coincident(1.0)
+
 
 if __name__ == "__main__":
     test()
+    integration_test()
