@@ -1,6 +1,13 @@
-"""AWSServiceBroker pulls and collects metadata from a bunch of AWS Services"""
-import os
+"""AWSServiceBroker pulls and collects metadata from a bunch of AWS Services
+   Note: This is the most complicated/messy class in the entire SageWorks project.
+      If you're looking at the class you're probably in the wrong place.
+      We suggest looking at these classes instead:
+      - api/meta.py (API for the data produced by this class)
+      - aws_service_broker/aws_service_connectors/*.py (Pulls the data from AWS)
+"""
+
 import sys
+import time
 import argparse
 from enum import Enum, auto
 import logging
@@ -9,6 +16,7 @@ from botocore.exceptions import ClientError
 
 # SageWorks Imports
 from sageworks.utils.sageworks_cache import SageWorksCache
+from sageworks.utils.config_manager import ConfigManager
 from sageworks.aws_service_broker.aws_service_connectors.s3_bucket import S3Bucket
 from sageworks.aws_service_broker.aws_service_connectors.glue_jobs import GlueJobs
 from sageworks.aws_service_broker.aws_service_connectors.data_catalog import DataCatalog
@@ -17,18 +25,14 @@ from sageworks.aws_service_broker.aws_service_connectors.model_registry import M
 from sageworks.aws_service_broker.aws_service_connectors.endpoints import Endpoints
 
 
-# SageWorks Logger
-log = logging.getLogger("sageworks")
-
-
 # Enumerated types for SageWorks Meta Requests
 class ServiceCategory(Enum):
     """Enumerated Types for SageWorks Meta Requests"""
 
     INCOMING_DATA_S3 = auto()
     GLUE_JOBS = auto()
-    DATA_SOURCES_S3 = auto()
-    FEATURE_SETS_S3 = auto()
+    # DATA_SOURCES_S3 = auto()  Memory Tests
+    # FEATURE_SETS_S3 = auto()  Memory Tests
     DATA_CATALOG = auto()
     FEATURE_STORE = auto()
     MODELS = auto()
@@ -38,31 +42,27 @@ class ServiceCategory(Enum):
 class AWSServiceBroker:
     """AWSServiceBroker pulls and collects metadata from a bunch of AWS Services"""
 
-    # Note: This database_scope is a list of databases that we want to pull metadata from
-    #       At some point, we should pull this from a config file.
-    database_scope = ["sageworks", "sagemaker_featurestore"]
-
     def __new__(cls):
         """AWSServiceBroker Singleton Pattern"""
         if not hasattr(cls, "instance"):
-            log.info("Creating the AWSServiceBroker Singleton...")
+            # Initialize class attributes here
+            cls.log = logging.getLogger("sageworks")
+            cls.database_scope = ["sageworks", "sagemaker_featurestore"]
+            cls.log.info("Creating the AWSServiceBroker Singleton...")
             cls.instance = super(AWSServiceBroker, cls).__new__(cls)
 
             # Class Initialization
-            cls.instance.__class_init__(cls.database_scope)
+            cls.instance.__class_init__()
 
         return cls.instance
 
     @classmethod
-    def __class_init__(cls, database_scope):
+    def __class_init__(cls):
         """AWSServiceBroker pulls and collects metadata from a bunch of AWS Services"""
-        cls.log = logging.getLogger("sageworks")
 
         # Grab our SageWorks Bucket
-        cls.sageworks_bucket = os.environ.get("SAGEWORKS_BUCKET")
-        if cls.sageworks_bucket is None:
-            print("Could not find ENV var for SAGEWORKS_BUCKET!")
-            sys.exit(1)
+        cm = ConfigManager()
+        cls.sageworks_bucket = cm.get_config("SAGEWORKS_BUCKET")
 
         # Construct bucket paths
         cls.incoming_data_bucket = "s3://" + cls.sageworks_bucket + "/incoming-data/"
@@ -78,10 +78,10 @@ class AWSServiceBroker:
 
         # Pull in AWS Service Connectors
         cls.incoming_data_s3 = S3Bucket(cls.incoming_data_bucket)
-        cls.data_sources_s3 = S3Bucket(cls.data_sources_bucket)
-        cls.feature_sets_s3 = S3Bucket(cls.feature_sets_bucket)
+        # cls.data_sources_s3 = S3Bucket(cls.data_sources_bucket)  Memory Tests
+        # cls.feature_sets_s3 = S3Bucket(cls.feature_sets_bucket)  Memory Tests
         cls.glue_jobs = GlueJobs()
-        cls.data_catalog = DataCatalog(database_scope)
+        cls.data_catalog = DataCatalog(cls.database_scope)
         cls.feature_store = FeatureStore()
         cls.model_registry = ModelRegistry()
         cls.endpoints = Endpoints()
@@ -99,8 +99,8 @@ class AWSServiceBroker:
         cls.connection_map = {
             ServiceCategory.INCOMING_DATA_S3: cls.incoming_data_s3,
             ServiceCategory.GLUE_JOBS: cls.glue_jobs,
-            ServiceCategory.DATA_SOURCES_S3: cls.data_sources_s3,
-            ServiceCategory.FEATURE_SETS_S3: cls.feature_sets_s3,
+            # ServiceCategory.DATA_SOURCES_S3: cls.data_sources_s3,  Memory Tests
+            # ServiceCategory.FEATURE_SETS_S3: cls.feature_sets_s3,  Memory Tests
             ServiceCategory.DATA_CATALOG: cls.data_catalog,
             ServiceCategory.FEATURE_STORE: cls.feature_store,
             ServiceCategory.MODELS: cls.model_registry,
@@ -114,14 +114,30 @@ class AWSServiceBroker:
         Args:
             category (ServiceCategory): The Category of metadata to Pull
         """
-        # Refresh the connection for the given category and pull new data
-        try:
-            cls.fresh_cache.set(category, True)
-            cls.connection_map[category].refresh()
-            cls.meta_cache.set(category, cls.connection_map[category].aws_meta())
-        except ClientError:
-            cls.log.warning(f"Failed to refresh AWS data for {category}")
-            cls.log.warning("Sometimes this happens when an artifact is being deleted/recreated...")
+        max_attempts = 5
+        sleep_times = [1, 2, 4, 8, 16]
+        for attempt in range(max_attempts):
+            try:
+                cls.fresh_cache.set(category, True)
+                cls.connection_map[category].refresh()
+                cls.meta_cache.set(category, cls.connection_map[category].summary())
+                return  # Success, exit the loop
+            except ClientError as error:
+                error_code = error.response["Error"]["Code"]
+                error_message = error.response["Error"]["Message"]
+                cls.log.warning(
+                    f"Attempt {attempt}: Failed to refresh AWS data for {category}: {error_code} - {error_message}"
+                )
+
+                # Exponential backoff for ThrottlingExceptions
+                if error_code == "ThrottlingException" and attempt < max_attempts:
+                    cls.log.warning(
+                        f"ThrottlingException: Waiting for {sleep_times[attempt]} seconds before retrying..."
+                    )
+                    time.sleep(sleep_times[attempt])
+                else:
+                    cls.log.warning(f"ClientError response: {error.response}")
+                    break  # Exit the loop
 
     @classmethod
     def get_metadata(cls, category: ServiceCategory, force_refresh: bool = False) -> dict:
@@ -150,7 +166,7 @@ class AWSServiceBroker:
 
         # Is the AWS data stale?
         if cls.fresh_cache.get(category) is None:
-            cls.log.debug(f"Async: Metadata for {category} is stale, launching refresh thread...")
+            cls.log.info(f"Async: Metadata for {category} is stale, launching refresh thread...")
             cls.fresh_cache.set(category, True)
 
             # Submit the data refresh task to the thread pool
@@ -229,11 +245,16 @@ if __name__ == "__main__":
     incoming_data_size = aws_broker.get_s3_object_sizes(ServiceCategory.INCOMING_DATA_S3)
     print(f"Incoming Data Size: {incoming_data_size} Bytes")
 
+    """Memory Tests
     data_sources_size = aws_broker.get_s3_object_sizes(ServiceCategory.DATA_SOURCES_S3)
     print(f"Data Sources Size: {data_sources_size} Bytes")
 
     abalone_size = aws_broker.get_s3_object_sizes(ServiceCategory.DATA_SOURCES_S3, prefix="abalone_data")
     print(f"Abalone Size: {abalone_size} Bytes")
+    """
+
+    # Get the Metadata for ALL the categories (with a force refresh)
+    pprint(aws_broker.get_all_metadata(force_refresh=True))
 
     # Wait for any open threads to finish
     aws_broker.shutdown()
