@@ -1,4 +1,5 @@
 """AthenaSource: SageWorks Data Source accessible through Athena"""
+
 import pandas as pd
 import awswrangler as wr
 from datetime import datetime
@@ -10,7 +11,7 @@ from pprint import pprint
 # SageWorks Imports
 from sageworks.core.artifacts.data_source_abstract import DataSourceAbstract
 from sageworks.aws_service_broker.aws_service_broker import ServiceCategory
-from sageworks.utils.iso_8601 import convert_all_to_iso8601
+from sageworks.utils.datetime_utils import convert_all_to_iso8601
 from sageworks.algorithms.sql import (
     sample_rows,
     value_counts,
@@ -20,7 +21,6 @@ from sageworks.algorithms.sql import (
     correlations,
 )
 from sageworks.utils.redis_cache import CustomEncoder
-from sageworks.utils.trace_calls import trace_calls
 from sageworks.utils.aws_utils import sageworks_meta_from_catalog_table_meta
 
 
@@ -28,10 +28,12 @@ class AthenaSource(DataSourceAbstract):
     """AthenaSource: SageWorks Data Source accessible through Athena
 
     Common Usage:
+        ```
         my_data = AthenaSource(data_uuid, database="sageworks")
         my_data.summary()
         my_data.details()
         df = my_data.query(f"select * from {data_uuid} limit 5")
+        ```
     """
 
     def __init__(self, data_uuid, database="sageworks", force_refresh: bool = False):
@@ -42,18 +44,21 @@ class AthenaSource(DataSourceAbstract):
             database (str): Athena Database Name (default: sageworks)
             force_refresh (bool): Force refresh of AWS Metadata (default: False)
         """
+        # Ensure the data_uuid is a valid name/id
+        self.ensure_valid_name(data_uuid)
 
         # Call superclass init
         super().__init__(data_uuid, database)
 
-        # Setup our AWS Broker catalog metadata
-        _catalog_meta = self.aws_broker.get_metadata(ServiceCategory.DATA_CATALOG, force_refresh=force_refresh)
-        try:
-            self.catalog_table_meta = _catalog_meta[self.get_database()].get(self.get_table_name())
-        except KeyError:
-            self.log.critical(f"Unable to find {self.get_database()} in Catalogs...")
-            self.log.critical("You must run the sageworks/aws_setup/aws_account_check.py script")
-            raise RuntimeError("Unable to find {self.get_database()} in Catalogs...")
+        # Flag for metadata cache refresh logic
+        self.metadata_refresh_needed = False
+
+        # Setup our AWS Metadata Broker
+        self.catalog_table_meta = self.meta_broker.data_source_details(
+            data_uuid, self.get_database(), refresh=force_refresh
+        )
+        if self.catalog_table_meta is None:
+            self.log.important(f"Unable to find {self.get_database()}:{self.get_table_name()} in Glue Catalogs...")
 
         # Call superclass post init
         super().__post_init__()
@@ -65,6 +70,7 @@ class AthenaSource(DataSourceAbstract):
         """Refresh our internal AWS Broker catalog metadata"""
         _catalog_meta = self.aws_broker.get_metadata(ServiceCategory.DATA_CATALOG, force_refresh=True)
         self.catalog_table_meta = _catalog_meta[self.get_database()].get(self.get_table_name())
+        self.metadata_refresh_needed = False
 
     def exists(self) -> bool:
         """Validation Checks for this Data Source"""
@@ -83,7 +89,6 @@ class AthenaSource(DataSourceAbstract):
         arn = f"arn:aws:glue:{region}:{account_id}:table/{self.get_database()}/{self.get_table_name()}"
         return arn
 
-    @trace_calls
     def sageworks_meta(self) -> dict:
         """Get the SageWorks specific metadata for this Artifact"""
 
@@ -97,6 +102,10 @@ class AthenaSource(DataSourceAbstract):
                 self.log.critical("Malformed Artifact! Delete this Artifact and recreate it!")
             return {}
 
+        # Check if we need to refresh our metadata
+        if self.metadata_refresh_needed:
+            self.refresh_meta()
+
         # Get the SageWorks Metadata from the Catalog Table Metadata
         return sageworks_meta_from_catalog_table_meta(self.catalog_table_meta)
 
@@ -125,6 +134,7 @@ class AthenaSource(DataSourceAbstract):
                 table=self.get_table_name(),
                 boto3_session=self.boto_session,
             )
+            self.metadata_refresh_needed = True
         except botocore.exceptions.ClientError as e:
             error_code = e.response["Error"]["Code"]
             if error_code == "InvalidInputException":
@@ -133,7 +143,7 @@ class AthenaSource(DataSourceAbstract):
                 self.log.error(new_meta)
             elif error_code == "ConcurrentModificationException":
                 self.log.warning("ConcurrentModificationException... trying again...")
-                time.sleep(1)
+                time.sleep(5)
                 wr.catalog.upsert_table_parameters(
                     parameters=new_meta,
                     database=self.get_database(),
@@ -155,7 +165,7 @@ class AthenaSource(DataSourceAbstract):
 
     def aws_url(self):
         """The AWS URL for looking at/querying this data source"""
-        sageworks_details = self.sageworks_meta().get("sageworks_details", "{}")
+        sageworks_details = self.sageworks_meta().get("sageworks_details", {})
         return sageworks_details.get("aws_url", "unknown")
 
     def created(self) -> datetime:
@@ -168,8 +178,10 @@ class AthenaSource(DataSourceAbstract):
 
     def num_rows(self) -> int:
         """Return the number of rows for this Data Source"""
-        count_df = self.query(f'select count(*) AS count from "{self.get_database()}"."{self.get_table_name()}"')
-        return count_df["count"][0]
+        count_df = self.query(
+            f'select count(*) AS sageworks_count from "{self.get_database()}"."{self.get_table_name()}"'
+        )
+        return count_df["sageworks_count"][0]
 
     def num_columns(self) -> int:
         """Return the number of columns for this Data Source"""
@@ -184,7 +196,14 @@ class AthenaSource(DataSourceAbstract):
         return [item["Type"] for item in self.catalog_table_meta["StorageDescriptor"]["Columns"]]
 
     def query(self, query: str) -> pd.DataFrame:
-        """Query the AthenaSource"""
+        """Query the AthenaSource
+
+        Args:
+            query (str): The query to run against the AthenaSource
+
+        Returns:
+            pd.DataFrame: The results of the query
+        """
         df = wr.athena.read_sql_query(
             sql=query,
             database=self.get_database(),
@@ -220,7 +239,7 @@ class AthenaSource(DataSourceAbstract):
 
     def athena_test_query(self):
         """Validate that Athena Queries are working"""
-        query = f"select count(*) as count from {self.get_table_name()}"
+        query = f"select count(*) as sageworks_count from {self.get_table_name()}"
         df = wr.athena.read_sql_query(
             sql=query,
             database=self.get_database(),
@@ -449,11 +468,20 @@ class AthenaSource(DataSourceAbstract):
 
         # Delete S3 Storage Objects (if they exist)
         try:
-            self.log.info(f"Deleting S3 Storage Object: {self.s3_storage_location()}...")
-            wr.s3.delete_objects(self.s3_storage_location(), boto3_session=self.boto_session)
+            # Make sure we add the trailing slash
+            s3_path = self.s3_storage_location()
+            s3_path = s3_path if s3_path.endswith("/") else f"{s3_path}/"
+
+            self.log.info(f"Deleting S3 Storage Objects: {s3_path}...")
+            wr.s3.delete_objects(s3_path, boto3_session=self.boto_session)
         except TypeError:
             self.log.warning("Malformed Artifact... good thing it's being deleted...")
 
+        # Delete any data in the Cache
+        for key in self.data_storage.list_subkeys(f"data_source:{self.uuid}:"):
+            self.log.info(f"Deleting Cache Key {key}...")
+            self.data_storage.delete(key)
+
 
 if __name__ == "__main__":
     """Exercise the AthenaSource Class"""
@@ -489,7 +517,7 @@ if __name__ == "__main__":
     print(f"Input: {my_data.get_input()}")
 
     # Get Tags associated with this Artifact
-    print(f"Tags: {my_data.sageworks_tags()}")
+    print(f"Tags: {my_data.get_tags()}")
 
     # Get a sample of the data
     my_df = my_data.sample()
@@ -547,7 +575,7 @@ if __name__ == "__main__":
     print("\n\nDisplay Columns")
     print(my_data.get_display_columns())
 
-    # Test an Data Source that doesn't exist
+    # Test a Data Source that doesn't exist
     # The rest of the tests are Disabled for now
     """
     print("\n\nTesting a Data Source that does not exist...")
