@@ -1,4 +1,5 @@
 """EndpointCore: SageWorks EndpointCore Class"""
+
 import time
 from datetime import datetime
 import botocore
@@ -6,21 +7,21 @@ import pandas as pd
 import numpy as np
 from io import StringIO
 import awswrangler as wr
-import ast
+from typing import Union
 import shap
+import joblib
 
 # Model Performance Scores
 from sklearn.metrics import (
     mean_absolute_error,
     r2_score,
-    mean_squared_error,
+    root_mean_squared_error,
     precision_recall_fscore_support,
     median_absolute_error,
     roc_auc_score,
     confusion_matrix,
 )
 from sklearn.preprocessing import LabelBinarizer
-from math import sqrt
 
 from sagemaker.serializers import CSVSerializer
 from sagemaker.deserializers import CSVDeserializer
@@ -38,20 +39,28 @@ class EndpointCore(Artifact):
     """EndpointCore: SageWorks EndpointCore Class
 
     Common Usage:
+        ```
         my_endpoint = EndpointCore(endpoint_uuid)
         prediction_df = my_endpoint.predict(test_df)
         metrics = my_endpoint.regression_metrics(target_column, prediction_df)
         for metric, value in metrics.items():
             print(f"{metric}: {value:0.3f}")
+        ```
     """
 
-    def __init__(self, endpoint_uuid, force_refresh: bool = False):
+    def __init__(self, endpoint_uuid, force_refresh: bool = False, legacy: bool = False):
         """EndpointCore Initialization
 
         Args:
             endpoint_uuid (str): Name of Endpoint in SageWorks
             force_refresh (bool, optional): Force a refresh of the AWS Broker. Defaults to False.
+            legacy (bool, optional): Force load of legacy models. Defaults to False.
         """
+
+        # Make sure the endpoint_uuid is a valid name
+        if not legacy:
+            self.ensure_valid_name(endpoint_uuid, delimiter="-")
+
         # Call SuperClass Initialization
         super().__init__(endpoint_uuid)
 
@@ -61,18 +70,28 @@ class EndpointCore(Artifact):
             self.endpoint_name
         )
 
-        # Sanity check and then set up our FeatureSet attributes
+        # Sanity check that we found the endpoint
         if self.endpoint_meta is None:
             self.log.important(f"Could not find endpoint {self.uuid} within current visibility scope")
             return
 
-        self.endpoint_return_columns = None
+        # Sanity check the Endpoint state
+        if self.endpoint_meta["EndpointStatus"] == "Failed":
+            self.log.critical(f"Endpoint {self.uuid} is in a failed state")
+            reason = self.endpoint_meta["FailureReason"]
+            self.log.critical(f"Failure Reason: {reason}")
+            self.log.critical("Please delete this endpoint and re-deploy...")
 
         # Set the Inference, Capture, and Monitoring S3 Paths
+        self.endpoint_inference_path = self.endpoints_s3_path + "/inference/" + self.uuid
+        self.endpoint_data_capture_path = self.endpoints_s3_path + "/data_capture/" + self.uuid
+        self.endpoint_monitoring_path = self.endpoints_s3_path + "/monitoring/" + self.uuid
+
+        # Set the Model Name
         self.model_name = self.get_input()
-        self.model_inference_path = self.models_s3_path + "/inference/" + self.model_name
-        self.model_data_capture_path = self.models_s3_path + "/data_capture/" + self.model_name
-        self.model_monitoring_path = self.models_s3_path + "/monitoring/" + self.model_name
+
+        # This is for endpoint error handling later
+        self.endpoint_return_columns = None
 
         # Call SuperClass Post Initialization
         super().__post_init__()
@@ -95,15 +114,24 @@ class EndpointCore(Artifact):
 
     def health_check(self) -> list[str]:
         """Perform a health check on this model
+
         Returns:
             list[str]: List of health issues
         """
+        if not self.ready():
+            return ["needs_onboard"]
+
         # Call the base class health check
         health_issues = super().health_check()
 
         # We're going to check for 5xx errors and no activity
         endpoint_metrics = self.endpoint_metrics()
 
+        # Check if we have metrics
+        if endpoint_metrics is None:
+            health_issues.append("unknown_error")
+            return health_issues
+
         # Check for 5xx errors
         num_errors = endpoint_metrics["Invocation5XXErrors"].sum()
         if num_errors > 5:
@@ -111,25 +139,30 @@ class EndpointCore(Artifact):
         elif num_errors > 0:
             health_issues.append("5xx_errors_min")
         else:
-            self.remove_sageworks_health_tag("5xx_errors")
-            self.remove_sageworks_health_tag("5xx_errors_min")
+            self.remove_health_tag("5xx_errors")
+            self.remove_health_tag("5xx_errors_min")
 
         # Check for Endpoint activity
         num_invocations = endpoint_metrics["Invocations"].sum()
         if num_invocations == 0:
             health_issues.append("no_activity")
         else:
-            self.remove_sageworks_health_tag("no_activity")
+            self.remove_health_tag("no_activity")
         return health_issues
 
-    def predict(self, feature_df: pd.DataFrame) -> pd.DataFrame:
+    def predict(self, eval_df: pd.DataFrame) -> pd.DataFrame:
         """Run inference/prediction on the given Feature DataFrame
         Args:
-            feature_df (pd.DataFrame): DataFrame to run predictions on (must have superset of features)
+            eval_df (pd.DataFrame): DataFrame to run predictions on (must have superset of features)
         Returns:
-            pd.DataFrame: Return the feature DataFrame with two additional columns (prediction, pred_proba)
+            pd.DataFrame: Return the DataFrame with additional columns, prediction and any _proba
         """
 
+        # Make sure the eval_df has the features used to train the model
+        features = ModelCore(self.model_name).features()
+        if features and not set(features).issubset(eval_df.columns):
+            raise ValueError(f"DataFrame does not contain required features: {features}")
+
         # Create our Endpoint Predictor Class
         predictor = Predictor(
             self.endpoint_name,
@@ -141,11 +174,11 @@ class EndpointCore(Artifact):
         # Now split up the dataframe into 500 row chunks, send those chunks to our
         # endpoint (with error handling) and stitch all the chunks back together
         df_list = []
-        for index in range(0, len(feature_df), 500):
+        for index in range(0, len(eval_df), 500):
             print("Processing...")
 
             # Compute partial DataFrames, add them to a list, and concatenate at the end
-            partial_df = self._endpoint_error_handling(predictor, feature_df[index : index + 500])
+            partial_df = self._endpoint_error_handling(predictor, eval_df[index : index + 500])
             df_list.append(partial_df)
 
         # Concatenate the dataframes
@@ -155,9 +188,14 @@ class EndpointCore(Artifact):
         # Note: Since we're using CSV serializers numeric columns often get changed to generic 'object' types
 
         # Hard Conversion
-        # Note: If are string/object columns we want to use 'ignore' here so those columns
-        #       won't raise an error (columns maintain current type)
-        converted_df = combined_df.apply(pd.to_numeric, errors="ignore")
+        # Note: We explicitly catch exceptions for columns that cannot be converted to numeric
+        converted_df = combined_df.copy()
+        for column in combined_df.columns:
+            try:
+                converted_df[column] = pd.to_numeric(combined_df[column])
+            except ValueError:
+                # If a ValueError is raised, the column cannot be converted to numeric, so we keep it as is
+                pass
 
         # Soft Conversion
         # Convert columns to the best possible dtype that supports the pd.NA missing value.
@@ -176,10 +214,13 @@ class EndpointCore(Artifact):
 
     def add_data_capture(self):
         """Add data capture to the endpoint"""
-        from sageworks.utils.model_monitoring import ModelMonitoring
+        self.get_monitor().add_data_capture()
 
-        mm = ModelMonitoring(self.endpoint_name)
-        mm.add_data_capture()
+    def get_monitor(self):
+        """Get the MonitorCore class for this endpoint"""
+        from sageworks.core.artifacts.monitor_core import MonitorCore
+
+        return MonitorCore(self.endpoint_name)
 
     def _endpoint_error_handling(self, predictor, feature_df):
         """Internal: Method that handles Errors, Retries, and Binary Search for Error Row(s)"""
@@ -258,10 +299,11 @@ class EndpointCore(Artifact):
         """Return the datetime when this artifact was last modified"""
         return self.endpoint_meta["LastModifiedTime"]
 
-    def endpoint_metrics(self) -> pd.DataFrame:
+    def endpoint_metrics(self) -> Union[pd.DataFrame, None]:
         """Return the metrics for this endpoint
+
         Returns:
-            pd.DataFrame: DataFrame with the metrics for this endpoint
+            pd.DataFrame: DataFrame with the metrics for this endpoint (or None if no metrics)
         """
 
         # Do we have it cached?
@@ -271,6 +313,8 @@ class EndpointCore(Artifact):
             return endpoint_metrics
 
         # We don't have it cached so let's get it from CloudWatch
+        if "ProductionVariants" not in self.endpoint_meta:
+            return None
         self.log.important("Updating endpoint metrics...")
         variant = self.endpoint_meta["ProductionVariants"][0]["VariantName"]
         endpoint_metrics = EndpointMetrics().get_metrics(self.uuid, variant=variant)
@@ -304,7 +348,10 @@ class EndpointCore(Artifact):
             details["instance_count"] = self.endpoint_meta["ProductionVariants"][0]["CurrentInstanceCount"] or "-"
         except KeyError:
             details["instance_count"] = "-"
-        details["variant"] = self.endpoint_meta["ProductionVariants"][0]["VariantName"]
+        if "ProductionVariants" in self.endpoint_meta:
+            details["variant"] = self.endpoint_meta["ProductionVariants"][0]["VariantName"]
+        else:
+            details["variant"] = "-"
 
         # Add the underlying model details
         details["model_name"] = self.model_name
@@ -312,7 +359,7 @@ class EndpointCore(Artifact):
         details["model_type"] = model_details.get("model_type", "unknown")
         details["model_metrics"] = model_details.get("model_metrics")
         details["confusion_matrix"] = model_details.get("confusion_matrix")
-        details["regression_predictions"] = model_details.get("regression_predictions")
+        details["predictions"] = model_details.get("predictions")
         details["inference_meta"] = model_details.get("inference_meta")
 
         # Add endpoint metrics from CloudWatch
@@ -324,14 +371,33 @@ class EndpointCore(Artifact):
         # Return the details
         return details
 
-    def make_ready(self) -> bool:
-        """This is a BLOCKING method that will wait until the Endpoint is ready
+    def onboard(self, interactive: bool = True) -> bool:
+        """This is a BLOCKING method that will onboard the Endpoint (make it ready)
+        Args:
+            interactive (bool, optional): If True, will prompt the user for information. (default: True)
         Returns:
-            bool: True if the Endpoint is ready, False otherwise
+            bool: True if the Endpoint is successfully onboarded, False otherwise
         """
-        self.set_status("ready")
-        self.remove_sageworks_health_tag("needs_onboard")
+        self.log.important(f"Onboarding {self.uuid}...")
+        self.set_status("onboarding")
+        self.remove_health_tag("needs_onboard")
+
+        # Make sure our input is defined
+        if self.get_input() == "unknown":
+            if interactive:
+                input_model = input("Input Model?: ")
+                self.upsert_sageworks_meta({"sageworks_input": input_model})
+                self.model_name = input_model
+            else:
+                self.log.error("Input Model is not defined!")
+                return False
+
+        # Run a health check and refresh the meta
+        time.sleep(2)  # Give the AWS Metadata a chance to update
+        self.health_check()
+        self.refresh_meta()
         self.details(recompute=True)
+        self.set_status("ready")
         return True
 
     def model_details(self) -> dict:
@@ -349,63 +415,126 @@ class EndpointCore(Artifact):
         """Return the type of model used in this Endpoint"""
         return self.details().get("model_type", "unknown")
 
-    def capture_performance_metrics(
-        self, feature_df: pd.DataFrame, target_column: str, data_name: str, data_hash: str, description: str
-    ) -> None:
-        """Capture the performance metrics for this Endpoint
+    def auto_inference(self, capture: bool = False) -> pd.DataFrame:
+        """Run inference on the endpoint using FeatureSet data
+
         Args:
-            feature_df (pd.DataFrame): DataFrame to run predictions on (must have superset of features)
-            target_column (str): Name of the target column
-            data_name (str): Name of the data used for inference
-            data_hash (str): Hash of the data used for inference
-            description (str): Description of the data used for inference
-        Returns:
-            None
-        Note:
-            This method captures performance metrics and writes them to the S3 Model Inference Folder
+            capture (bool, optional): Capture the inference results and metrics (default=False)
         """
 
-        # Run predictions on the feature_df
-        prediction_df = self.predict(feature_df)
+        # This Utility needs to be loaded now to avoid circular imports
+        from sageworks.utils.endpoint_utils import fs_evaluation_data
 
-        # Compute the metrics
+        eval_data_df = fs_evaluation_data(self)
+        capture_uuid = "training_holdout" if capture else None
+        return self.inference(eval_data_df, capture_uuid)
+
+    def inference(self, eval_df: pd.DataFrame, capture_uuid: str = None) -> pd.DataFrame:
+        """Run inference and compute performance metrics with optional capture
+
+        Args:
+            eval_df (pd.DataFrame): DataFrame to run predictions on (must have superset of features)
+            capture_uuid (str, optional): UUID of the inference capture (default=None)
+
+        Returns:
+            pd.DataFrame: DataFrame with the inference results
+
+        Note:
+            If capture=True inference/performance metrics are written to S3 Endpoint Inference Folder
+        """
+
+        # Run predictions on the evaluation data
+        prediction_df = self.predict(eval_df)
+
+        # Get the target column
+        target_column = ModelCore(self.model_name).target()
+
+        # Sanity Check that the target column is present
+        if target_column not in prediction_df.columns:
+            self.log.warning(f"Target Column {target_column} not found in prediction_df!")
+            self.log.warning("In order to compute metrics, the target column must be present!")
+            return prediction_df
+
+        # Compute the standard performance metrics for this model
         model_type = self.model_type()
         if model_type == ModelType.REGRESSOR.value:
+            prediction_df = self.residuals(target_column, prediction_df)
             metrics = self.regression_metrics(target_column, prediction_df)
         elif model_type == ModelType.CLASSIFIER.value:
             metrics = self.classification_metrics(target_column, prediction_df)
         else:
-            raise ValueError(f"Unknown Model Type: {model_type}")
+            # Unknown Model Type: Give log message and set metrics to empty dataframe
+            self.log.warning(f"Unknown Model Type: {model_type}")
+            metrics = pd.DataFrame()
+
+        # Print out the metrics
+        print(f"Performance Metrics for {self.model_name} on {self.uuid}")
+        print(metrics.head())
+
+        # Capture the inference results and metrics
+        if capture_uuid is not None:
+            description = capture_uuid.replace("_", " ").title()
+            self._capture_inference_results(capture_uuid, prediction_df, target_column, metrics, description)
+
+        # Return the prediction DataFrame
+        return prediction_df
+
+    def _capture_inference_results(
+        self,
+        capture_uuid: str,
+        pred_results_df: pd.DataFrame,
+        target_column: str,
+        metrics: pd.DataFrame,
+        description: str,
+    ):
+        """Internal: Capture the inference results and metrics to S3
+
+        Args:
+            capture_uuid (str): UUID of the inference capture
+            pred_results_df (pd.DataFrame): DataFrame with the prediction results
+            target_column (str): Name of the target column
+            metrics (pd.DataFrame): DataFrame with the performance metrics
+            description (str): Description of the inference results
+        """
+
+        # Compute a dataframe hash (just use the last 8)
+        data_hash = joblib.hash(pred_results_df)[:8]
 
         # Metadata for the model inference
         inference_meta = {
-            "test_data": data_name,
-            "test_data_hash": data_hash,
-            "test_rows": len(feature_df),
+            "name": capture_uuid,
+            "data_hash": data_hash,
+            "num_rows": len(pred_results_df),
             "description": description,
         }
 
+        # Create the S3 Path for the Inference Capture
+        inference_capture_path = f"{self.endpoint_inference_path}/{capture_uuid}"
+
         # Write the metadata dictionary, and metrics to our S3 Model Inference Folder
         wr.s3.to_json(
             pd.DataFrame([inference_meta]),
-            f"{self.model_inference_path}/inference_meta.json",
+            f"{inference_capture_path}/inference_meta.json",
             index=False,
         )
-        self.log.debug(f"Writing metrics to {self.model_inference_path}/inference_metrics.csv")
-        wr.s3.to_csv(metrics, f"{self.model_inference_path}/inference_metrics.csv", index=False)
+        self.log.info(f"Writing metrics to {inference_capture_path}/inference_metrics.csv")
+        wr.s3.to_csv(metrics, f"{inference_capture_path}/inference_metrics.csv", index=False)
 
-        # Write the confusion matrix to our S3 Model Inference Folder
+        # Write the predictions to our S3 Model Inference Folder (just the target and prediction columns)
+        self.log.info(f"Writing predictions to {inference_capture_path}/inference_predictions.csv")
+        prediction_col = "prediction" if "prediction" in pred_results_df.columns else "predictions"
+        output_columns = [target_column, prediction_col]
+        output_columns += [col for col in pred_results_df.columns if col.endswith("_proba")]
+        subset_df = pred_results_df[output_columns]
+        wr.s3.to_csv(subset_df, f"{inference_capture_path}/inference_predictions.csv", index=False)
+
+        # CLASSIFIER: Write the confusion matrix to our S3 Model Inference Folder
+        model_type = self.model_type()
         if model_type == ModelType.CLASSIFIER.value:
-            conf_mtx = self.confusion_matrix(target_column, prediction_df)
-            self.log.debug(f"Writing confusion matrix to {self.model_inference_path}/inference_cm.csv")
+            conf_mtx = self.confusion_matrix(target_column, pred_results_df)
+            self.log.info(f"Writing confusion matrix to {inference_capture_path}/inference_cm.csv")
             # Note: Unlike other dataframes here, we want to write the index (labels) to the CSV
-            wr.s3.to_csv(conf_mtx, f"{self.model_inference_path}/inference_cm.csv", index=True)
-
-        # Write the regression predictions to our S3 Model Inference Folder
-        if model_type == ModelType.REGRESSOR.value:
-            pred_df = self.regression_predictions(target_column, prediction_df)
-            self.log.debug(f"Writing reg predictions to {self.model_inference_path}/inference_predictions.csv")
-            wr.s3.to_csv(pred_df, f"{self.model_inference_path}/inference_predictions.csv", index=False)
+            wr.s3.to_csv(conf_mtx, f"{inference_capture_path}/inference_cm.csv", index=True)
 
         #
         # Generate SHAP values for our Prediction Dataframe
@@ -416,33 +545,39 @@ class EndpointCore(Artifact):
 
         # Get the exact features used to train the model
         model_features = model_artifact.feature_names_in_
-        X_pred = prediction_df[model_features]
+        X_pred = pred_results_df[model_features]
 
         # Compute the SHAP values
-        shap_vals = self.shap_values(model_artifact, X_pred)
+        try:
+            shap_vals = self.shap_values(model_artifact, X_pred)
 
-        # Multiple shap vals CSV for classifiers
-        if model_type == ModelType.CLASSIFIER.value:
-            # Need a separate shapley values CSV for each class
-            for i, class_shap_vals in enumerate(shap_vals):
-                df_shap = pd.DataFrame(class_shap_vals, columns=X_pred.columns)
+            # Multiple shap vals CSV for classifiers
+            if model_type == ModelType.CLASSIFIER.value:
+                # Need a separate shapley values CSV for each class
+                for i, class_shap_vals in enumerate(shap_vals):
+                    df_shap = pd.DataFrame(class_shap_vals, columns=X_pred.columns)
+
+                    # Write shap vals to S3 Model Inference Folder
+                    shap_file_path = f"{inference_capture_path}/inference_shap_values_class_{i}.csv"
+                    self.log.info(f"Writing SHAP values to {shap_file_path}")
+                    wr.s3.to_csv(df_shap, shap_file_path, index=False)
+
+            # Single shap vals CSV for regressors
+            if model_type == ModelType.REGRESSOR.value:
+                # Format shap values into single dataframe
+                df_shap = pd.DataFrame(shap_vals, columns=X_pred.columns)
 
                 # Write shap vals to S3 Model Inference Folder
-                self.log.debug(f"Writing SHAP values to {self.model_inference_path}/inference_shap_values.csv")
-                wr.s3.to_csv(df_shap, f"{self.model_inference_path}/inference_shap_values_class_{i}.csv", index=False)
+                self.log.debug(f"Writing SHAP values to {inference_capture_path}/inference_shap_values.csv")
+                wr.s3.to_csv(df_shap, f"{inference_capture_path}/inference_shap_values.csv", index=False)
 
-        # Single shap vals CSV for regressors
-        if model_type == ModelType.REGRESSOR.value:
-            # Format shap values into single dataframe
-            df_shap = pd.DataFrame(shap_vals, columns=X_pred.columns)
-
-            # Write shap vals to S3 Model Inference Folder
-            self.log.debug(f"Writing SHAP values to {self.model_inference_path}/inference_shap_values.csv")
-            wr.s3.to_csv(df_shap, f"{self.model_inference_path}/inference_shap_values.csv", index=False)
+        except Exception as e:
+            self.log.error(f"Error computing SHAP values: {e}")
 
         # Now recompute the details for our Model
         self.log.important(f"Recomputing Details for {self.model_name} to show latest Inference Results...")
         model = ModelCore(self.model_name)
+        model._load_inference_metrics(capture_uuid)
         model.details(recompute=True)
 
         # Recompute the details so that inference model metrics are updated
@@ -452,9 +587,11 @@ class EndpointCore(Artifact):
     @staticmethod
     def shap_values(model, X: pd.DataFrame) -> np.array:
         """Compute the SHAP values for this Model
+
         Args:
             model (Model): Model object
             X (pd.DataFrame): DataFrame with the prediction results
+
         Returns:
             pd.DataFrame: DataFrame with the SHAP values
         """
@@ -475,21 +612,52 @@ class EndpointCore(Artifact):
 
         # Compute the metrics
         y_true = prediction_df[target_column]
-        y_pred = prediction_df["prediction"]
+        prediction_col = "prediction" if "prediction" in prediction_df.columns else "predictions"
+        y_pred = prediction_df[prediction_col]
 
         mae = mean_absolute_error(y_true, y_pred)
-        rmse = sqrt(mean_squared_error(y_true, y_pred))
+        rmse = root_mean_squared_error(y_true, y_pred)
         r2 = r2_score(y_true, y_pred)
         # Mean Absolute Percentage Error
         mape = np.mean(np.where(y_true != 0, np.abs((y_true - y_pred) / y_true), np.abs(y_true - y_pred))) * 100
         # Median Absolute Error
         medae = median_absolute_error(y_true, y_pred)
 
-        # Return the metrics
-        return pd.DataFrame.from_records([{"MAE": mae, "RMSE": rmse, "R2": r2, "MAPE": mape, "MedAE": medae}])
+        # Organize and return the metrics
+        metrics = {
+            "MAE": round(mae, 3),
+            "RMSE": round(rmse, 3),
+            "R2": round(r2, 3),
+            "MAPE": round(mape, 3),
+            "MedAE": round(medae, 3),
+            "NumRows": len(prediction_df),
+        }
+        return pd.DataFrame.from_records([metrics])
 
-    @staticmethod
-    def classification_metrics(target_column: str, prediction_df: pd.DataFrame) -> pd.DataFrame:
+    def residuals(self, target_column: str, prediction_df: pd.DataFrame) -> pd.DataFrame:
+        """Add the residuals to the prediction DataFrame
+        Args:
+            target_column (str): Name of the target column
+            prediction_df (pd.DataFrame): DataFrame with the prediction results
+        Returns:
+            pd.DataFrame: DataFrame with two new columns called 'residuals' and 'residuals_abs'
+        """
+        # Sanity Check that this is a regression model
+        if self.model_type() != ModelType.REGRESSOR.value:
+            self.log.warning("Residuals are only computed for regression models")
+            return prediction_df
+
+        # Compute the residuals
+        y_true = prediction_df[target_column]
+        prediction_col = "prediction" if "prediction" in prediction_df.columns else "predictions"
+        y_pred = prediction_df[prediction_col]
+
+        # Add the residuals and the absolute values to the DataFrame
+        prediction_df["residuals"] = y_true - y_pred
+        prediction_df["residuals_abs"] = np.abs(y_true - y_pred)
+        return prediction_df
+
+    def classification_metrics(self, target_column: str, prediction_df: pd.DataFrame) -> pd.DataFrame:
         """Compute the performance metrics for this Endpoint
         Args:
             target_column (str): Name of the target column
@@ -502,8 +670,9 @@ class EndpointCore(Artifact):
         labels = prediction_df[target_column].unique()
 
         # Calculate scores
+        prediction_col = "prediction" if "prediction" in prediction_df.columns else "predictions"
         scores = precision_recall_fscore_support(
-            prediction_df[target_column], prediction_df["prediction"], average=None, labels=labels
+            prediction_df[target_column], prediction_df[prediction_col], average=None, labels=labels
         )
 
         # Calculate ROC AUC
@@ -511,16 +680,23 @@ class EndpointCore(Artifact):
         # - A value of 0.5 indicates no discrimination (equivalent to random guessing)
         # - A score close to 1 indicates high discriminative power
 
-        # Convert 'pred_proba' column to a 2D NumPy array
-        y_score = np.array([ast.literal_eval(x) for x in prediction_df["pred_proba"]], dtype=float)
-        # y_score = np.array(prediction_df['pred_proba'].tolist())
+        # Sanity check for older versions that have a single column for probability
+        if "pred_proba" in prediction_df.columns:
+            self.log.error("Older version of prediction output detected, rerun inference...")
+            roc_auc = [0.0] * len(labels)
 
-        # One-hot encode the true labels
-        lb = LabelBinarizer()
-        lb.fit(prediction_df[target_column])
-        y_true = lb.transform(prediction_df[target_column])
+        # Convert probability columns to a 2D NumPy array
+        else:
+            proba_columns = [col for col in prediction_df.columns if col.endswith("_proba")]
+            y_score = prediction_df[proba_columns].to_numpy()
 
-        roc_auc = roc_auc_score(y_true, y_score, multi_class="ovr", average=None)
+            # One-hot encode the true labels
+            lb = LabelBinarizer()
+            lb.fit(prediction_df[target_column])
+            y_true = lb.transform(prediction_df[target_column])
+
+            # Compute ROC AUC
+            roc_auc = roc_auc_score(y_true, y_score, multi_class="ovr", average=None)
 
         # Put the scores into a dataframe
         score_df = pd.DataFrame(
@@ -548,7 +724,8 @@ class EndpointCore(Artifact):
         """
 
         y_true = prediction_df[target_column]
-        y_pred = prediction_df["prediction"]
+        prediction_col = "prediction" if "prediction" in prediction_df.columns else "predictions"
+        y_pred = prediction_df[prediction_col]
 
         # Compute the confusion matrix
         conf_mtx = confusion_matrix(y_true, y_pred)
@@ -560,23 +737,29 @@ class EndpointCore(Artifact):
         conf_mtx_df = pd.DataFrame(conf_mtx, index=labels, columns=labels)
         return conf_mtx_df
 
-    def regression_predictions(self, target: str, prediction_df: pd.DataFrame) -> pd.DataFrame:
-        """Compute the regression predictions for this Endpoint
-        Args:
-            target (str): Name of the target column
-            prediction_df (pd.DataFrame): DataFrame with the prediction results
-        Returns:
-            pd.DataFrame: DataFrame with the regression predictions
-        """
-
-        # Return the predictions
-        return prediction_df[[target, "prediction"]]
-
     def endpoint_config_name(self) -> str:
         # Grab the Endpoint Config Name from the AWS
         details = self.sm_client.describe_endpoint(EndpointName=self.endpoint_name)
         return details["EndpointConfigName"]
 
+    def set_input(self, input: str, force=False):
+        """Override: Set the input data for this artifact
+
+        Args:
+            input (str): Name of input for this artifact
+            force (bool, optional): Force the input to be set. Defaults to False.
+        Note:
+            We're going to not allow this to be used for Models
+        """
+        if not force:
+            self.log.warning(f"Endpoint {self.uuid}: Does not allow manual override of the input!")
+            return
+
+        # Okay we're going to allow this to be set
+        self.log.important(f"{self.uuid}: Setting input to {input}...")
+        self.log.important("Be careful with this! It breaks automatic provenance of the artifact!")
+        self.upsert_sageworks_meta({"sageworks_input": input})
+
     def delete(self):
         """Delete an existing Endpoint: Underlying Models, Configuration, and Endpoint"""
         self.delete_endpoint_models()
@@ -596,19 +779,32 @@ class EndpointCore(Artifact):
             self.log.info(f"Deleting Endpoint Monitoring Schedule {schedule['MonitoringScheduleName']}...")
             self.sm_client.delete_monitoring_schedule(MonitoringScheduleName=schedule["MonitoringScheduleName"])
 
+        # Delete any inference, data_capture or monitoring artifacts
+        for s3_path in [self.endpoint_inference_path, self.endpoint_data_capture_path, self.endpoint_monitoring_path]:
+
+            # Make sure we add the trailing slash
+            s3_path = s3_path if s3_path.endswith("/") else f"{s3_path}/"
+            objects = wr.s3.list_objects(s3_path, boto3_session=self.boto_session)
+            for obj in objects:
+                self.log.info(f"Deleting S3 Object {obj}...")
+            wr.s3.delete_objects(objects, boto3_session=self.boto_session)
+
+        # Now delete any data in the Cache
+        for key in self.data_storage.list_subkeys(f"endpoint:{self.uuid}:"):
+            self.log.info(f"Deleting Cache Key: {key}")
+            self.data_storage.delete(key)
+
         # Okay now delete the Endpoint
         try:
-            time.sleep(1)  # Let AWS catch up with any deletions performed above
+            time.sleep(2)  # Let AWS catch up with any deletions performed above
             self.log.info(f"Deleting Endpoint {self.uuid}...")
             self.sm_client.delete_endpoint(EndpointName=self.uuid)
         except botocore.exceptions.ClientError as e:
             self.log.info("Endpoint ClientError...")
             raise e
 
-        # Now delete any data in the Cache
-        for key in self.data_storage.list_subkeys(f"endpoint:{self.uuid}"):
-            self.log.info(f"Deleting Cache Key: {key}")
-            self.data_storage.delete(key)
+        # One more sleep to let AWS fully register the endpoint deletion
+        time.sleep(5)
 
     def delete_endpoint_models(self):
         """Delete the underlying Model for an Endpoint"""
@@ -625,14 +821,20 @@ class EndpointCore(Artifact):
         model_names = [variant["ModelName"] for variant in endpoint_config["ProductionVariants"]]
         for model_name in model_names:
             self.log.info(f"Deleting Model {model_name}...")
-            self.sm_client.delete_model(ModelName=model_name)
+            try:
+                self.sm_client.delete_model(ModelName=model_name)
+            except botocore.exceptions.ClientError as error:
+                error_code = error.response["Error"]["Code"]
+                error_message = error.response["Error"]["Message"]
+                if error_code == "ResourceInUse":
+                    self.log.warning(f"Model {model_name} is still in use...")
+                else:
+                    self.log.warning(f"Error: {error_code} - {error_message}")
 
 
 if __name__ == "__main__":
     """Exercise the Endpoint Class"""
-    from sageworks.core.transforms.pandas_transforms.features_to_pandas import (
-        FeaturesToPandas,
-    )
+    from sageworks.utils.endpoint_utils import fs_evaluation_data
 
     # Grab an EndpointCore object and pull some information from it
     my_endpoint = EndpointCore("abalone-regression-end")
@@ -645,7 +847,7 @@ if __name__ == "__main__":
     print(my_endpoint.modified())
 
     # Get the tags associated with this Endpoint
-    print(f"Tags: {my_endpoint.sageworks_tags()}")
+    print(f"Tags: {my_endpoint.get_tags()}")
 
     print("Details:")
     print(f"{my_endpoint.details(recompute=True)}")
@@ -653,65 +855,23 @@ if __name__ == "__main__":
     # Serverless?
     print(f"Serverless: {my_endpoint.is_serverless()}")
 
-    # Temp Testing
-    from sageworks.core.artifacts.feature_set_core import FeatureSetCore
+    # Health Check
+    print(f"Health Check: {my_endpoint.health_check()}")
 
-    end = EndpointCore("abalone-regression-end")
-    model = end.get_input()
-    feature_set = ModelCore(model).get_input()
-    features = FeatureSetCore(feature_set)
-    table = features.get_training_view_table()
-    df = features.query(f"SELECT * FROM {table} where training = 0")
-    DATA_NAME = "Test Data (20) 2023_08_31"
-    DATA_HASH = "12345"
-    DESCRIPTION = "Test Features"
-    TARGET_COLUMN = "class_number_of_rings"
+    # Run Auto Inference on the Endpoint (uses the FeatureSet)
+    print("Running Auto Inference...")
+    my_endpoint.auto_inference()
 
-    # Capture the performance metrics for this Endpoint
-    end.capture_performance_metrics(
-        df, TARGET_COLUMN, data_name=DATA_NAME, data_hash=DATA_HASH, description=DESCRIPTION
-    )
+    # Run Inference where we provide the data
+    # Note: This dataframe could be from a FeatureSet or any other source
+    print("Running Inference...")
+    eval_df = fs_evaluation_data(my_endpoint)
+    pred_results = my_endpoint.inference(eval_df)
 
-    #
-    # This section is all about INFERENCE TESTING
-    #
-    INFERENCE_TESTING = True
-    if INFERENCE_TESTING:
-        REGRESSION = True
-        if REGRESSION:
-            my_endpoint = EndpointCore("abalone-regression-end")
-            feature_to_pandas = FeaturesToPandas("abalone_features")
-            my_target_column = "class_number_of_rings"
-            data_name = ("abalone_holdout_2023_10_19",)
-            data_hash = ("12345",)
-            description = "Test Abalone Data"
-        else:
-            my_endpoint = EndpointCore("wine-classification-end")
-            feature_to_pandas = FeaturesToPandas("wine_features")
-            my_target_column = "wine_class"
-            data_name = ("wine_holdout_2023_10_19",)
-            data_hash = ("67890",)
-            description = "Test Wine Data"
+    # Now set capture=True to save inference results and metrics
+    eval_df = fs_evaluation_data(my_endpoint)
+    pred_results = my_endpoint.inference(eval_df, capture_uuid="holdout_xyz")
 
-        # Transform the DataSource into a Pandas DataFrame (with max_rows = 500)
-        feature_to_pandas.transform(max_rows=500)
-
-        # Grab the output and show it
-        my_feature_df = feature_to_pandas.get_output()
-        print(my_feature_df)
-
-        # Okay now run inference against our Features DataFrame
-        my_prediction_df = my_endpoint.predict(my_feature_df)
-        print(my_prediction_df)
-
-        # Compute performance metrics for out test predictions
-        if REGRESSION:
-            my_metrics = my_endpoint.regression_metrics(my_target_column, my_prediction_df)
-        else:
-            my_metrics = my_endpoint.classification_metrics(my_target_column, my_prediction_df)
-        print(my_metrics)
-
-        # Capture the performance metrics for this Endpoint
-        my_endpoint.capture_performance_metrics(
-            my_feature_df, my_target_column, data_name=data_name, data_hash=data_hash, description=description
-        )
+    # Run Inference and metrics for a Classification Endpoint
+    class_endpoint = EndpointCore("wine-classification-end")
+    class_endpoint.auto_inference()
