@@ -1,4 +1,5 @@
 """FeatureSet: SageWorks Feature Set accessible through Athena"""
+
 import time
 from datetime import datetime, timezone
 from typing import Union
@@ -22,9 +23,11 @@ class FeatureSetCore(Artifact):
     """FeatureSetCore: SageWorks FeatureSetCore Class
 
     Common Usage:
+        ```
         my_features = FeatureSetCore(feature_uuid)
         my_features.summary()
         my_features.details()
+        ```
     """
 
     def __init__(self, feature_set_uuid: str, force_refresh: bool = False):
@@ -35,6 +38,9 @@ class FeatureSetCore(Artifact):
             force_refresh (bool): Force a refresh of the Feature Set metadata (default: False)
         """
 
+        # Make sure the feature_set name is valid
+        self.ensure_valid_name(feature_set_uuid)
+
         # Call superclass init
         super().__init__(feature_set_uuid)
 
@@ -183,8 +189,18 @@ class FeatureSetCore(Artifact):
         """Return the number of rows of the internal DataSource"""
         return self.data_source.num_rows()
 
-    def query(self, query: str) -> pd.DataFrame:
-        """Query the internal DataSource"""
+    def query(self, query: str, overwrite: bool = True) -> pd.DataFrame:
+        """Query the internal DataSource
+
+        Args:
+            query (str): The query to run against the DataSource
+            overwrite (bool): Overwrite the table name in the query (default: True)
+
+        Returns:
+            pd.DataFrame: The results of the query
+        """
+        if overwrite:
+            query = query.replace(" " + self.uuid + " ", " " + self.athena_table + " ")
         return self.data_source.query(query)
 
     def aws_url(self):
@@ -306,7 +322,7 @@ class FeatureSetCore(Artifact):
         # Additional Details
         details["sageworks_status"] = self.get_status()
         details["sageworks_input"] = self.get_input()
-        details["sageworks_tags"] = ":".join(self.sageworks_tags())
+        details["sageworks_tags"] = self.tag_delimiter.join(self.get_tags())
 
         # Underlying Storage Details
         details["storage_type"] = "athena"  # TODO: Add RDS support
@@ -338,12 +354,12 @@ class FeatureSetCore(Artifact):
         self.delete_training_view()
 
         # Feature Sets can often have a lot of cruft so delete the entire bucket/prefix
-        s3_delete_path = self.feature_sets_s3_path + f"/{self.uuid}"
+        s3_delete_path = self.feature_sets_s3_path + f"/{self.uuid}/"
         self.log.info(f"Deleting All FeatureSet S3 Storage Objects {s3_delete_path}")
         wr.s3.delete_objects(s3_delete_path, boto3_session=self.boto_session)
 
         # Now delete any data in the Cache
-        for key in self.data_storage.list_subkeys(f"feature_set:{self.uuid}"):
+        for key in self.data_storage.list_subkeys(f"feature_set:{self.uuid}:"):
             self.log.info(f"Deleting Cache Key: {key}")
             self.data_storage.delete(key)
 
@@ -372,15 +388,21 @@ class FeatureSetCore(Artifact):
         view_name = f"{self.athena_table}_training"
         self.log.important(f"Creating default Training View {view_name}...")
 
-        # Construct the CREATE VIEW query with random assignment
-        create_view_query = f"""
-        CREATE OR REPLACE VIEW {view_name} AS
-        SELECT *, CASE
-            WHEN RAND() < 0.8 THEN 1  -- Assign roughly 80% to training
-            ELSE 0  -- Assign roughly 20% to hold-out
-        END AS training
-        FROM {self.athena_table}
-        """
+        # Do we already have a training column?
+        if "training" in self.column_names():
+            create_view_query = f"CREATE OR REPLACE VIEW {view_name} AS SELECT * FROM {self.athena_table}"
+        else:
+            # No training column, so create one:
+            #    Construct the CREATE VIEW query with a simple modulo operation for the 80/20 split
+            #    using self.record_id as the stable identifier for row numbering
+            create_view_query = f"""
+            CREATE OR REPLACE VIEW {view_name} AS
+            SELECT *, CASE
+                WHEN MOD(ROW_NUMBER() OVER (ORDER BY {self.record_id}), 10) < 8 THEN 1  -- Assign 80% to training
+                ELSE 0  -- Assign roughly 20% to validation
+            END AS training
+            FROM {self.athena_table}
+            """
 
         # Execute the CREATE VIEW query
         self.data_source.execute_statement(create_view_query)
@@ -416,12 +438,38 @@ class FeatureSetCore(Artifact):
         # Execute the CREATE VIEW query
         self.data_source.execute_statement(create_view_query)
 
+    def set_hold_out_ids(self, id_column: str, hold_out_ids: list[str]):
+        """Set the hold out ids for this FeatureSet
+
+        Args:
+            id_column (str): The name of the id column in the output DataFrame.
+            hold_out_ids (list[str]): The list of hold out ids.
+        """
+        self.create_training_view(id_column, hold_out_ids)
+
+    def get_hold_out_ids(self, id_column: str) -> list[str]:
+        """Get the hold out ids for this FeatureSet
+
+        Args:
+            id_column (str): The name of the id column in the output DataFrame.
+
+        Returns:
+            list[str]: The list of hold out ids.
+        """
+        training_view_table = self.get_training_view_table(create=False)
+        if training_view_table is not None:
+            query = f"SELECT {id_column} FROM {training_view_table} WHERE training = 0"
+            hold_out_ids = self.query(query)[id_column].tolist()
+            return hold_out_ids
+        else:
+            return []
+
     def get_training_view_table(self, create: bool = True) -> Union[str, None]:
         """Get the name of the training view for this FeatureSet
         Args:
             create (bool): Create the training view if it doesn't exist (default=True)
         Returns:
-            str: The name of the training view for this FeatureSet (or None if it doesn't exist)
+            str: The name of the training view for this FeatureSet
         """
         training_view_name = f"{self.athena_table}_training"
         glue_client = self.boto_session.client("glue")
@@ -431,18 +479,26 @@ class FeatureSetCore(Artifact):
         except glue_client.exceptions.EntityNotFoundException:
             if not create:
                 return None
-            self.log.warning(f"Training View for {self.uuid} doesn't exist, creating a default one...")
+            self.log.warning(f"Training View for {self.uuid} doesn't exist, creating one...")
             self.create_default_training_view()
             time.sleep(1)  # Give AWS a second to catch up
             return training_view_name
 
     def delete_training_view(self):
         """Delete the training view for this FeatureSet"""
-        training_view_table = self.get_training_view_table()
-        if training_view_table is not None:
-            self.log.info(f"Deleting Training View {training_view_table} for {self.uuid}")
-            glue_client = self.boto_session.client("glue")
-            glue_client.delete_table(DatabaseName=self.athena_database, Name=training_view_table)
+        try:
+            training_view_table = self.get_training_view_table(create=False)
+            if training_view_table is not None:
+                self.log.info(f"Deleting Training View {training_view_table} for {self.uuid}")
+                glue_client = self.boto_session.client("glue")
+                glue_client.delete_table(DatabaseName=self.athena_database, Name=training_view_table)
+        except botocore.exceptions.ClientError as error:
+            # For ResourceNotFound/ValidationException, this is fine, otherwise raise all other exceptions
+            if error.response["Error"]["Code"] in ["ResourceNotFound", "ValidationException"]:
+                self.log.warning(f"Training View for {self.uuid} doesn't exist, nothing to delete...")
+                pass
+            else:
+                raise error
 
     def descriptive_stats(self, recompute: bool = False) -> dict:
         """Get the descriptive stats for the numeric columns of the underlying DataSource
@@ -552,22 +608,29 @@ class FeatureSetCore(Artifact):
         # Okay now call/return the DataSource ready() method
         return self.data_source.ready()
 
-    def make_ready(self) -> bool:
-        """This is a BLOCKING method that will wait until the FeatureSet is ready"""
+    def onboard(self) -> bool:
+        """This is a BLOCKING method that will onboard the FeatureSet (make it ready)"""
 
-        # Call our underlying DataSource make_ready method
+        # Set our status to onboarding
+        self.log.important(f"Onboarding {self.uuid}...")
+        self.set_status("onboarding")
+        self.remove_health_tag("needs_onboard")
+
+        # Call our underlying DataSource onboard method
         self.data_source.refresh_meta()
         if not self.data_source.exists():
             self.log.critical(f"Data Source check failed for {self.uuid}")
             self.log.critical("Delete this Feature Set and recreate it to fix this issue")
             return False
         if not self.data_source.ready():
-            self.data_source.make_ready()
+            self.data_source.onboard()
 
-        # Set ourselves to ready
-        self.set_status("ready")
-        self.remove_sageworks_health_tag("needs_onboard")
+        # Run a health check and refresh the meta
+        time.sleep(2)  # Give the AWS Metadata a chance to update
+        self.health_check()
+        self.refresh_meta()
         self.details(recompute=True)
+        self.set_status("ready")
         return True
 
 
@@ -602,7 +665,7 @@ if __name__ == "__main__":
 
     # Get the metadata and tags associated with this feature set
     print(f"SageWorks Meta: {my_features.sageworks_meta()}")
-    print(f"SageWorks Tags: {my_features.sageworks_tags()}")
+    print(f"SageWorks Tags: {my_features.get_tags()}")
 
     # Get a summary for this Feature Set
     print("\nSummary:")
@@ -636,12 +699,20 @@ if __name__ == "__main__":
     my_features.create_default_training_view()
 
     # Test the hold out set functionality with ints
-    print("Setting hold out ids (ints)...")
+    print("Setting hold out ids...")
     table = my_features.get_training_view_table()
     df = my_features.query(f"SELECT id, name FROM {table}")
     my_hold_out_ids = [id for id in df["id"] if id < 20]
     my_features.create_training_view("id", my_hold_out_ids)
 
+    # Convenience methods to set and get the hold out ids
+    print("Setting hold out ids...")
+    my_features.set_hold_out_ids("id", my_hold_out_ids)
+    print("Getting hold out ids...")
+    hold_output = my_features.get_hold_out_ids("id")
+    print(hold_output)
+    assert set(hold_output) == set(my_hold_out_ids)
+
     # Test the hold out set functionality with strings
     print("Setting hold out ids (strings)...")
     my_hold_out_ids = [name for name in df["name"] if int(name.split(" ")[1]) > 80]
