@@ -1,4 +1,5 @@
 """ModelCore: SageWorks ModelCore Class"""
+
 import time
 from datetime import datetime
 import urllib.parse
@@ -8,6 +9,7 @@ import botocore
 
 import pandas as pd
 import awswrangler as wr
+from urllib.parse import urlparse
 from awswrangler.exceptions import NoFilesFound
 from sagemaker import TrainingJobAnalytics
 from sagemaker.model import Model as SagemakerModel
@@ -15,6 +17,7 @@ from sagemaker.model import Model as SagemakerModel
 # SageWorks Imports
 from sageworks.core.artifacts.artifact import Artifact
 from sageworks.aws_service_broker.aws_service_broker import ServiceCategory
+from sageworks.utils.aws_utils import newest_files, pull_s3_data
 
 
 # Enumerated Model Types
@@ -32,18 +35,28 @@ class ModelCore(Artifact):
     """ModelCore: SageWorks ModelCore Class
 
     Common Usage:
+        ```
         my_model = ModelCore(model_uuid)
         my_model.summary()
         my_model.details()
+        ```
     """
 
-    def __init__(self, model_uuid: str, force_refresh: bool = False, model_type: ModelType = None):
+    def __init__(
+        self, model_uuid: str, force_refresh: bool = False, model_type: ModelType = None, legacy: bool = False
+    ):
         """ModelCore Initialization
         Args:
             model_uuid (str): Name of Model in SageWorks.
             force_refresh (bool, optional): Force a refresh of the AWS Broker. Defaults to False.
             model_type (ModelType, optional): Set this for newly created Models. Defaults to None.
+            legacy (bool, optional): Force load of legacy models. Defaults to False.
         """
+
+        # Make sure the model name is valid
+        if not legacy:
+            self.ensure_valid_name(model_uuid, delimiter="-")
+
         # Call SuperClass Initialization
         super().__init__(model_uuid)
 
@@ -55,6 +68,7 @@ class ModelCore(Artifact):
             self.log.important(f"Could not find model {self.model_name} within current visibility scope")
             self.latest_model = None
             self.model_type = ModelType.UNKNOWN
+            return
         else:
             try:
                 self.latest_model = self.model_meta[0]
@@ -68,10 +82,13 @@ class ModelCore(Artifact):
                 self.log.critical(f"Model {self.model_name} appears to be malformed. Delete and recreate it!")
                 self.latest_model = None
                 self.model_type = ModelType.UNKNOWN
+                return
 
-        # Set the Model Training S3 Paths
+        # Set the Model Training S3 Path
         self.model_training_path = self.models_s3_path + "/training/" + self.model_name
-        self.model_inference_path = self.models_s3_path + "/inference/" + self.model_name
+
+        # Get our Endpoint Inference Path (might be None)
+        self.endpoint_inference_path = self.get_endpoint_inference_path()
 
         # Call SuperClass Post Initialization
         super().__post_init__()
@@ -105,17 +122,18 @@ class ModelCore(Artifact):
         if self._get_model_type() == ModelType.UNKNOWN:
             health_issues.append("model_type_unknown")
         else:
-            self.remove_sageworks_health_tag("model_type_unknown")
+            self.remove_health_tag("model_type_unknown")
 
-        # Model Metrics
-        if self.model_metrics() is None:
+        # Model Performance Metrics
+        if self.performance_metrics() is None:
             health_issues.append("metrics_needed")
         else:
-            self.remove_sageworks_health_tag("metrics_needed")
+            self.remove_health_tag("metrics_needed")
         return health_issues
 
     def latest_model_object(self) -> SagemakerModel:
         """Return the latest AWS Sagemaker Model object for this SageWorks Model
+
         Returns:
            sagemaker.model.Model: AWS Sagemaker Model object
         """
@@ -123,50 +141,109 @@ class ModelCore(Artifact):
             model_data=self.model_package_arn(), sagemaker_session=self.sm_session, image_uri=self.model_image()
         )
 
-    def model_metrics(self) -> Union[pd.DataFrame, None]:
-        """Retrieve the training metrics for this model
+    def list_inference_runs(self) -> list[str]:
+        """List the inference runs for this model
+
+        Returns:
+            list[str]: List of inference run UUIDs
+        """
+        if self.endpoint_inference_path is None:
+            return ["model_training"]  # Just the training run
+        directories = wr.s3.list_directories(path=self.endpoint_inference_path + "/")
+        inference_runs = [urlparse(directory).path.split("/")[-2] for directory in directories]
+
+        # We're going to add the training to the front of the list
+        inference_runs.insert(0, "model_training")
+        return inference_runs
+
+    def performance_metrics(self, capture_uuid: str = "latest") -> Union[pd.DataFrame, None]:
+        """Retrieve the performance metrics for this model
+
+        Args:
+            capture_uuid (str, optional): Specific capture_uuid or "training" (default: "latest")
         Returns:
             pd.DataFrame: DataFrame of the Model Metrics
+
+        Note:
+            If a capture_uuid isn't specified this will try to return something reasonable
         """
-        # Grab the metrics from the SageWorks Metadata (try inference first, then training)
-        metrics = self._pull_inference_metrics()
-        if metrics is not None:
-            return metrics
-        metrics = self.sageworks_meta().get("sageworks_training_metrics")
-        return pd.DataFrame.from_dict(metrics) if isinstance(metrics, dict) else None
+        # Try to get the auto_capture 'training_holdout' or the training
+        if capture_uuid == "latest":
+            metrics_df = self.performance_metrics("training_holdout")
+            return metrics_df if metrics_df is not None else self.performance_metrics("model_training")
 
-    def model_shapley_values(self) -> Union[list[pd.DataFrame], pd.DataFrame, None]:
-        # Shapley only available from inference at the moment, training may come later
-        df_shap = self._pull_shapley_values()
-        return df_shap
+        # Grab the metrics captured during model training (could return None)
+        if capture_uuid == "model_training":
+            metrics = self.sageworks_meta().get("sageworks_training_metrics")
+            return pd.DataFrame.from_dict(metrics) if metrics else None
 
-    def confusion_matrix(self) -> Union[pd.DataFrame, None]:
+        else:  # Specific capture_uuid (could return None)
+            s3_path = f"{self.endpoint_inference_path}/{capture_uuid}/inference_metrics.csv"
+            metrics = pull_s3_data(s3_path, embedded_index=True)
+            if metrics is not None:
+                return metrics
+            else:
+                self.log.warning(f"Performance metrics {capture_uuid} not found for {self.model_name}!")
+                return None
+
+    def confusion_matrix(self, capture_uuid: str = "latest") -> Union[pd.DataFrame, None]:
         """Retrieve the confusion_matrix for this model
+
+        Args:
+            capture_uuid (str, optional): Specific capture_uuid or "training" (default: "latest")
         Returns:
             pd.DataFrame: DataFrame of the Confusion Matrix (might be None)
         """
-        # Grab the confusion matrix from the SageWorks Metadata
-        cm = self._pull_inference_cm()
-        if cm is not None:
-            return cm
-        cm = self.sageworks_meta().get("sageworks_training_cm")
-        return pd.DataFrame.from_dict(cm) if cm else None
+        # Grab the metrics from the SageWorks Metadata (try inference first, then training)
+        if capture_uuid == "latest":
+            cm = self.sageworks_meta().get("sageworks_inference_cm")
+            return cm if cm is not None else self.confusion_matrix("model_training")
 
-    def regression_predictions(self) -> Union[pd.DataFrame, None]:
-        """Retrieve the regression based predictions for this model
+        # Grab the confusion matrix captured during model training (could return None)
+        if capture_uuid == "model_training":
+            cm = self.sageworks_meta().get("sageworks_training_cm")
+            return pd.DataFrame.from_dict(cm) if cm else None
+
+        else:  # Specific capture_uuid
+            s3_path = f"{self.endpoint_inference_path}/{capture_uuid}/inference_cm.csv"
+            cm = pull_s3_data(s3_path, embedded_index=True)
+            if cm is not None:
+                return cm
+            else:
+                self.log.warning(f"Confusion Matrix {capture_uuid} not found for {self.model_name}!")
+                return None
+
+    def predictions(self, capture_uuid: str = "training_holdout") -> Union[pd.DataFrame, None]:
+        """Retrieve the predictions for this model
+
+        Args:
+            capture_uuid (str, optional): Specific capture_uuid or "training" (default: "training_holdout")
         Returns:
-            pd.DataFrame: DataFrame of the Regression based Predictions (might be None)
+            pd.DataFrame: DataFrame of the Predictions (might be None)
         """
+        # Grab the metrics from the SageWorks Metadata (try inference first, then training)
+        inference_preds = self.inference_predictions(capture_uuid)
+        if inference_preds is not None:
+            return inference_preds
+        return self.validation_predictions()
 
-        # Pull the regression predictions, try first from inference, then from training
-        s3_path = f"{self.model_inference_path}/inference_predictions.csv"
-        df = self._pull_s3_model_artifacts(s3_path)
-        if df is not None:
-            return df
-        else:
-            s3_path = f"{self.model_training_path}/validation_predictions.csv"
-            df = self._pull_s3_model_artifacts(s3_path)
-            return df
+    def set_input(self, input: str, force: bool = False):
+        """Override: Set the input data for this artifact
+
+        Args:
+            input (str): Name of input for this artifact
+            force (bool, optional): Force the input to be set (default: False)
+        Note:
+            We're going to not allow this to be used for Models
+        """
+        if not force:
+            self.log.warning(f"Model {self.uuid}: Does not allow manual override of the input!")
+            return
+
+        # Okay we're going to allow this to be set
+        self.log.important(f"{self.uuid}: Setting input to {input}...")
+        self.log.important("Be careful with this! It breaks automatic provenance of the artifact!")
+        self.upsert_sageworks_meta({"sageworks_input": input})
 
     def size(self) -> float:
         """Return the size of this data in MegaBytes"""
@@ -189,11 +266,11 @@ class ModelCore(Artifact):
         return self.latest_model["ModelPackageArn"]
 
     def model_container_info(self) -> dict:
-        """Containiner Info for the Latest Model Package"""
+        """Container Info for the Latest Model Package"""
         return self.latest_model["ModelPackageDetails"]["InferenceSpecification"]["Containers"][0]
 
     def model_image(self) -> str:
-        """Containiner Image for the Latest Model Package"""
+        """Container Image for the Latest Model Package"""
         return self.model_container_info()["Image"]
 
     def aws_url(self):
@@ -208,6 +285,82 @@ class ModelCore(Artifact):
         """Return the datetime when this artifact was last modified"""
         return self.latest_model["CreationTime"]
 
+    def register_endpoint(self, endpoint_name: str):
+        """Add this endpoint to the set of registered endpoints for the model
+
+        Args:
+            endpoint_name (str): Name of the endpoint
+        """
+        self.log.important(f"Registering Endpoint {endpoint_name} with Model {self.uuid}...")
+        registered_endpoints = set(self.sageworks_meta().get("sageworks_registered_endpoints", []))
+        registered_endpoints.add(endpoint_name)
+        self.upsert_sageworks_meta({"sageworks_registered_endpoints": list(registered_endpoints)})
+
+        # A new endpoint means we need to refresh our inference path
+        time.sleep(2)  # Give the AWS Metadata a chance to update
+        self.endpoint_inference_path = self.get_endpoint_inference_path()
+
+    def get_endpoints(self) -> list[str]:
+        """Get the list of registered endpoints for this Model
+
+        Returns:
+            list[str]: List of registered endpoints
+        """
+        return self.sageworks_meta().get("sageworks_registered_endpoints", [])
+
+    def get_endpoint_inference_path(self) -> str:
+        """Get the S3 Path for the Inference Data"""
+
+        # Look for any Registered Endpoints
+        registered_endpoints = self.sageworks_meta().get("sageworks_registered_endpoints")
+
+        # Note: We may have 0 to N endpoints, so we find the one with the most recent artifacts
+        if registered_endpoints:
+            endpoint_inference_base = self.endpoints_s3_path + "/inference/"
+            endpoint_inference_paths = [endpoint_inference_base + e for e in registered_endpoints]
+            return newest_files(endpoint_inference_paths, self.sm_session)
+        else:
+            self.log.warning(f"No registered endpoints found for {self.model_name}!")
+            return None
+
+    def target(self) -> Union[str, None]:
+        """Return the target for this Model (if supervised, else None)
+
+        Returns:
+            str: Target column for this Model (if supervised, else None)
+        """
+        return self.sageworks_meta().get("sageworks_model_target")  # Returns None if not found
+
+    def features(self) -> Union[list[str], None]:
+        """Return a list of features used for this Model
+
+        Returns:
+            list[str]: List of features used for this Model
+        """
+        return self.sageworks_meta().get("sageworks_model_features")  # Returns None if not found
+
+    def class_labels(self) -> Union[list[str], None]:
+        """Return the class labels for this Model (if it's a classifier)
+
+        Returns:
+            list[str]: List of class labels
+        """
+        if self.model_type == ModelType.CLASSIFIER:
+            return self.sageworks_meta().get("class_labels")  # Returns None if not found
+        else:
+            return None
+
+    def set_class_labels(self, labels: list[str]):
+        """Return the class labels for this Model (if it's a classifier)
+
+        Args:
+            labels (list[str]): List of class labels
+        """
+        if self.model_type == ModelType.CLASSIFIER:
+            self.upsert_sageworks_meta({"class_labels": labels})
+        else:
+            self.log.error(f"Model {self.model_name} is not a classifier!")
+
     def details(self, recompute=False) -> dict:
         """Additional Details about this Model
         Args:
@@ -244,19 +397,16 @@ class ModelCore(Artifact):
         details["transform_types"] = inference_spec["SupportedTransformInstanceTypes"]
         details["content_types"] = inference_spec["SupportedContentTypes"]
         details["response_types"] = inference_spec["SupportedResponseMIMETypes"]
-        details["model_metrics"] = self.model_metrics()
+        details["model_metrics"] = self.performance_metrics()
         if self.model_type == ModelType.CLASSIFIER:
             details["confusion_matrix"] = self.confusion_matrix()
-            details["regression_predictions"] = None
+            details["predictions"] = None
         else:
             details["confusion_matrix"] = None
-            details["regression_predictions"] = self.regression_predictions()
-
-        # Set Shapley values
-        details["shapley_values"] = self.model_shapley_values()
+            details["predictions"] = self.predictions()
 
         # Grab the inference metadata
-        details["inference_meta"] = self._pull_inference_metadata()
+        details["inference_meta"] = self.inference_metadata()
 
         # Cache the details
         self.data_storage.set(storage_key, details)
@@ -272,20 +422,6 @@ class ModelCore(Artifact):
         # Our current list of expected metadata, we can add to this as needed
         return ["sageworks_status", "sageworks_training_metrics", "sageworks_training_cm"]
 
-    def onboard(self) -> bool:
-        """Onboard this Model into SageWorks
-        Returns:
-            bool: True if the Model was successfully onboarded, False otherwise
-        """
-
-        # Determine the Model Type
-        while self.is_model_unknown():
-            self._determine_model_type()
-
-        # Call the superclass onboard
-        super().onboard()
-        return True
-
     def is_model_unknown(self) -> bool:
         """Is the Model Type unknown?"""
         return self.model_type == ModelType.UNKNOWN
@@ -305,18 +441,68 @@ class ModelCore(Artifact):
             self.log.warning(f"Unknown Model Type {model_type}!")
             self._set_model_type(ModelType.UNKNOWN)
 
-    def make_ready(self) -> bool:
-        """This is a BLOCKING method that will wait until the Model is ready
+    def onboard(self, interactive: bool = True) -> bool:
+        """This is a BLOCKING method that will onboard the Model (make it ready)
+
+        Args:
+            interactive (bool, optional): If True, will prompt the user for information. Defaults to True.
         Returns:
-            bool: True if the Model is ready, False otherwise
+            bool: True if the Model is successfully onboarded, False otherwise
         """
-        self._pull_training_job_metrics(force_pull=True)
-        self.set_status("ready")
-        self.remove_sageworks_health_tag("needs_onboard")
-        time.sleep(1)  # Give the AWS Metadata a chance to update
+        # Set the status to onboarding
+        self.set_status("onboarding")
+
+        # Interactive Stuff
+        if interactive:
+            # Determine the Model Type
+            while self.is_model_unknown():
+                self._determine_model_type()
+
+            # Determine the Target Column (can be None)
+            if self.target() is None:
+                target_column = input("Target Column? (for unsupervised/transformer just type None): ")
+                if target_column in ["None", "none", ""]:
+                    target_column = None
+                self.upsert_sageworks_meta({"sageworks_model_target": target_column})
+
+            # Determine the Feature Columns
+            if self.features() is None:
+                feature_columns = input("Feature Columns? (use commas): ")
+                feature_columns = [e.strip() for e in feature_columns.split(",")]
+                if feature_columns not in [["None"], ["none"], [""]]:
+                    self.upsert_sageworks_meta({"sageworks_model_features": feature_columns})
+
+            # Registered Endpoints?
+            if not self.get_endpoints():
+                endpoints = input("Register Endpoints? (use commas for multiple): ")
+                endpoints = [e.strip() for e in endpoints.split(",")]
+                if endpoints not in [["None"], ["none"], [""]]:
+                    for endpoint in endpoints:
+                        self.log.info(f"Registering Endpoint {endpoint}...")
+                        self.register_endpoint(endpoint)
+
+            # Model Owner?
+            if self.get_owner() in [None, "unknown"]:
+                owner = input("Model Owner: ")
+                if owner in ["None", "none", ""]:
+                    self.set_owner("unknown")
+                else:
+                    self.set_owner(owner)
+
+        # Load the training metrics and inference metrics
+        self._load_training_metrics()
+        self._load_inference_metrics()
+        self._load_inference_cm()
+
+        # Remove the needs_onboard tag
+        self.remove_health_tag("needs_onboard")
+
+        # Run a health check and refresh the meta
+        time.sleep(2)  # Give the AWS Metadata a chance to update
         self.health_check()
         self.refresh_meta()
         self.details(recompute=True)
+        self.set_status("ready")
         return True
 
     def delete(self):
@@ -336,18 +522,13 @@ class ModelCore(Artifact):
         self.log.info(f"Deleting Model Group {self.model_name}...")
         self.sm_client.delete_model_package_group(ModelPackageGroupName=self.model_name)
 
-        # Delete any inference artifacts
-        s3_delete_path = f"{self.model_inference_path}"
+        # Delete any training artifacts
+        s3_delete_path = f"{self.model_training_path}/"
         self.log.info(f"Deleting Training S3 Objects {s3_delete_path}")
         wr.s3.delete_objects(s3_delete_path, boto3_session=self.boto_session)
 
-        # Delete any training artifacts
-        s3_delete_path = f"{self.model_training_path}"
-        self.log.info(f"Deleting Inference S3 Objects {s3_delete_path}")
-        wr.s3.delete_objects(s3_delete_path, boto3_session=self.boto_session)
-
         # Delete any data in the Cache
-        for key in self.data_storage.list_subkeys(f"model:{self.uuid}"):
+        for key in self.data_storage.list_subkeys(f"model:{self.uuid}:"):
             self.log.info(f"Deleting Cache Key {key}...")
             self.data_storage.delete(key)
 
@@ -355,7 +536,7 @@ class ModelCore(Artifact):
         """Internal: Set the Model Type for this Model"""
         self.model_type = model_type
         self.upsert_sageworks_meta({"sageworks_model_type": self.model_type.value})
-        self.remove_sageworks_health_tag("model_type_unknown")
+        self.remove_health_tag("model_type_unknown")
 
     def _get_model_type(self) -> ModelType:
         """Internal: Query the SageWorks Metadata to get the model type
@@ -372,105 +553,32 @@ class ModelCore(Artifact):
             self.log.warning(f"Could not determine model type for {self.model_name}!")
             return ModelType.UNKNOWN
 
-    def _pull_inference_metadata(self) -> Union[pd.DataFrame, None]:
-        """Internal: Retrieve the inference metadata for this model
-        Returns:
-            dict: Dictionary of the inference metadata (might be None)
+    def _load_training_metrics(self):
+        """Internal: Retrieve the training metrics and Confusion Matrix for this model
+                     and load the data into the SageWorks Metadata
+
         Notes:
-            Basically when the inference was run, name of the dataset, the MD5, etc
+            This may or may not exist based on whether we have access to TrainingJobAnalytics
         """
-        s3_path = f"{self.model_inference_path}/inference_meta.json"
-        try:
-            return wr.s3.read_json(s3_path)
-        except NoFilesFound:
-            self.log.info(f"Could not find model inference meta at {s3_path}...")
-            return None
-
-    def _pull_shapley_values(self) -> Union[list[pd.DataFrame], pd.DataFrame, None]:
-        """Internal: Retrieve the inference Shapely values for this model
-        Returns:
-            pd.DataFrame: Dataframe of the shapley values for the prediction dataframe
-        """
-
-        # Multiple CSV if classifier
-        if self.model_type == ModelType.CLASSIFIER:
-            # CSVs for shap values are indexed by prediction class
-            # Because we don't know how many classes there are, we need to search through
-            # a list of S3 objects in the parent folder
-            s3_paths = wr.s3.list_objects(self.model_inference_path)
-            return [
-                self._pull_s3_model_artifacts(f, embedded_index=False) for f in s3_paths if "inference_shap_values" in f
-            ]
-
-        # One CSV if regressor
-        if self.model_type == ModelType.REGRESSOR:
-            s3_path = f"{self.model_inference_path}/inference_shap_values.csv"
-            return self._pull_s3_model_artifacts(s3_path, embedded_index=False)
-
-    def _pull_inference_metrics(self) -> Union[pd.DataFrame, None]:
-        """Internal: Retrieve the inference model metrics for this model
-        Returns:
-            pd.DataFrame: DataFrame of the inference model metrics (might be None)
-        """
-        s3_path = f"{self.model_inference_path}/inference_metrics.csv"
-        return self._pull_s3_model_artifacts(s3_path)
-
-    def _pull_inference_cm(self) -> Union[pd.DataFrame, None]:
-        """Internal: Retrieve the inference Confusion Matrix for this model
-        Returns:
-            pd.DataFrame: DataFrame of the inference Confusion Matrix (might be None)
-        """
-        s3_path = f"{self.model_inference_path}/inference_cm.csv"
-        return self._pull_s3_model_artifacts(s3_path, embedded_index=True)
-
-    def _pull_s3_model_artifacts(self, s3_path, embedded_index=False) -> Union[pd.DataFrame, None]:
-        """Internal: Helper method to pull Model Artifact data from S3 storage
-        Args:
-            s3_path (str): S3 Path to the Model Artifact
-            embedded_index (bool, optional): Is the index embedded in the CSV? Defaults to False.
-        Returns:
-            pd.DataFrame: DataFrame of the Model Artifact (metrics, CM, regression_preds) (might be None)
-        """
-
-        # Pull the CSV file from S3
-        try:
-            if embedded_index:
-                df = wr.s3.read_csv(s3_path, index_col=0)
-            else:
-                df = wr.s3.read_csv(s3_path)
-            return df
-        except NoFilesFound:
-            self.log.info(f"Could not find model artifact at {s3_path}...")
-            return None
-
-    def _pull_training_job_metrics(self, force_pull=False):
-        """Internal: Grab any captured metrics from the training job for this model
-        Args:
-            force_pull (bool, optional): Force a pull from TrainingJobAnalytics. Defaults to False.
-        """
-
-        # First check if we have already computed the various metrics
-        model_metrics = self.sageworks_meta().get("sageworks_training_metrics")
-        if self.model_type == ModelType.REGRESSOR:
-            if model_metrics and not force_pull:
-                return
-
-        # For classifiers, we need to pull the confusion matrix as well
-        cm = self.sageworks_meta().get("sageworks_training_cm")
-        if model_metrics and cm and not force_pull:
-            return
-
-        # We don't have them, so go and grab the training job metrics
-        self.log.info(f"Pulling training job metrics for {self.training_job_name}...")
         try:
             df = TrainingJobAnalytics(training_job_name=self.training_job_name).dataframe()
+            if df.empty:
+                self.log.warning(f"No training job metrics found for {self.training_job_name}")
+                self.upsert_sageworks_meta({"sageworks_training_metrics": None, "sageworks_training_cm": None})
+                return
             if self.model_type == ModelType.REGRESSOR:
                 if "timestamp" in df.columns:
                     df = df.drop(columns=["timestamp"])
 
+                # We're going to pivot the DataFrame to get the desired structure
+                reg_metrics_df = df.set_index("metric_name").T
+
                 # Store and return the metrics in the SageWorks Metadata
-                self.upsert_sageworks_meta({"sageworks_training_metrics": df.to_dict(), "sageworks_training_cm": None})
+                self.upsert_sageworks_meta(
+                    {"sageworks_training_metrics": reg_metrics_df.to_dict(), "sageworks_training_cm": None}
+                )
                 return
+
         except (KeyError, botocore.exceptions.ClientError):
             self.log.warning(f"No training job metrics found for {self.training_job_name}")
             # Store and return the metrics in the SageWorks Metadata
@@ -486,6 +594,104 @@ class ModelCore(Artifact):
                 {"sageworks_training_metrics": metrics_df.to_dict(), "sageworks_training_cm": cm_df.to_dict()}
             )
 
+    def _load_inference_metrics(self, capture_uuid: str = "training_holdout"):
+        """Internal: Retrieve the inference model metrics for this model
+                     and load the data into the SageWorks Metadata
+
+        Args:
+            capture_uuid (str, optional): A specific capture_uuid (default: "training_holdout")
+        Notes:
+            This may or may not exist based on whether an Endpoint ran Inference
+        """
+        s3_path = f"{self.endpoint_inference_path}/{capture_uuid}/inference_metrics.csv"
+        inference_metrics = pull_s3_data(s3_path)
+
+        # Store data into the SageWorks Metadata
+        metrics_storage = None if inference_metrics is None else inference_metrics.to_dict("records")
+        self.upsert_sageworks_meta({"sageworks_inference_metrics": metrics_storage})
+
+    def _load_inference_cm(self, capture_uuid: str = "training_holdout"):
+        """Internal: Pull the inference Confusion Matrix for this model
+                     and load the data into the SageWorks Metadata
+
+        Args:
+            capture_uuid (str, optional): A specific capture_uuid (default: "training_holdout")
+
+        Returns:
+            pd.DataFrame: DataFrame of the inference Confusion Matrix (might be None)
+
+        Notes:
+            This may or may not exist based on whether an Endpoint ran Inference
+        """
+        s3_path = f"{self.endpoint_inference_path}/{capture_uuid}/inference_cm.csv"
+        inference_cm = pull_s3_data(s3_path, embedded_index=True)
+
+        # Store data into the SageWorks Metadata
+        cm_storage = None if inference_cm is None else inference_cm.to_dict("records")
+        self.upsert_sageworks_meta({"sageworks_inference_cm": cm_storage})
+
+    def inference_metadata(self, capture_uuid: str = "training_holdout") -> Union[pd.DataFrame, None]:
+        """Retrieve the inference metadata for this model
+
+        Args:
+            capture_uuid (str, optional): A specific capture_uuid (default: "training_holdout")
+
+        Returns:
+            dict: Dictionary of the inference metadata (might be None)
+        Notes:
+            Basically when Endpoint inference was run, name of the dataset, the MD5, etc
+        """
+        # Sanity check the inference path (which may or may not exist)
+        if self.endpoint_inference_path is None:
+            return None
+
+        # Check for model_training capture_uuid
+        if capture_uuid == "model_training":
+            # Create a DataFrame with the training metadata
+            meta_df = pd.DataFrame(
+                [
+                    {
+                        "name": "AWS Training Capture",
+                        "data_hash": "N/A",
+                        "num_rows": "-",
+                        "description": "-",
+                    }
+                ]
+            )
+            return meta_df
+
+        # Pull the inference metadata
+        try:
+            s3_path = f"{self.endpoint_inference_path}/{capture_uuid}/inference_meta.json"
+            return wr.s3.read_json(s3_path)
+        except NoFilesFound:
+            self.log.info(f"Could not find model inference meta at {s3_path}...")
+            return None
+
+    def inference_predictions(self, capture_uuid: str = "training_holdout") -> Union[pd.DataFrame, None]:
+        """Retrieve the captured prediction results for this model
+
+        Args:
+            capture_uuid (str, optional): Specific capture_uuid (default: training_holdout)
+
+        Returns:
+            pd.DataFrame: DataFrame of the Captured Predictions (might be None)
+        """
+        self.log.important(f"Grabbing {capture_uuid} predictions for {self.model_name}...")
+        s3_path = f"{self.endpoint_inference_path}/{capture_uuid}/inference_predictions.csv"
+        return pull_s3_data(s3_path)
+
+    def validation_predictions(self) -> Union[pd.DataFrame, None]:
+        """Internal: Retrieve the captured prediction results for this model
+
+        Returns:
+            pd.DataFrame: DataFrame of the Captured Validation Predictions (might be None)
+        """
+        self.log.important(f"Grabbing Validation Predictions for {self.model_name}...")
+        s3_path = f"{self.model_training_path}/validation_predictions.csv"
+        df = pull_s3_data(s3_path)
+        return df
+
     def _extract_training_job_name(self) -> Union[str, None]:
         """Internal: Extract the training job name from the ModelDataUrl"""
         try:
@@ -529,6 +735,39 @@ class ModelCore(Artifact):
 
         return metrics_df, cm_df
 
+    def shapley_values(self, capture_uuid: str = "training_holdout") -> Union[list[pd.DataFrame], pd.DataFrame, None]:
+        """Retrieve the Shapely values for this model
+
+        Args:
+            capture_uuid (str, optional): Specific capture_uuid (default: training_holdout)
+
+        Returns:
+            pd.DataFrame: Dataframe of the shapley values for the prediction dataframe
+
+        Notes:
+            This may or may not exist based on whether an Endpoint ran Shapley
+        """
+
+        # Sanity check the inference path (which may or may not exist)
+        if self.endpoint_inference_path is None:
+            return None
+
+        # Construct the S3 path for the Shapley values
+        shapley_s3_path = f"{self.endpoint_inference_path}/{capture_uuid}"
+
+        # Multiple CSV if classifier
+        if self.model_type == ModelType.CLASSIFIER:
+            # CSVs for shap values are indexed by prediction class
+            # Because we don't know how many classes there are, we need to search through
+            # a list of S3 objects in the parent folder
+            s3_paths = wr.s3.list_objects(shapley_s3_path)
+            return [pull_s3_data(f) for f in s3_paths if "inference_shap_values" in f]
+
+        # One CSV if regressor
+        if self.model_type == ModelType.REGRESSOR:
+            s3_path = f"{shapley_s3_path}/inference_shap_values.csv"
+            return pull_s3_data(s3_path)
+
 
 if __name__ == "__main__":
     """Exercise the ModelCore Class"""
@@ -542,14 +781,14 @@ if __name__ == "__main__":
     print(f"Model Check: {my_model.exists()}")
 
     # Make sure the model is 'ready'
-    my_model.make_ready()
+    my_model.onboard(interactive=False)
 
     # Get the ARN of the Model Group
     print(f"Model Group ARN: {my_model.group_arn()}")
     print(f"Model Package ARN: {my_model.arn()}")
 
     # Get the tags associated with this Model
-    print(f"Tags: {my_model.sageworks_tags()}")
+    print(f"Tags: {my_model.get_tags()}")
 
     # Get the SageWorks metadata associated with this Model
     print(f"SageWorks Meta: {my_model.sageworks_meta()}")
@@ -560,20 +799,23 @@ if __name__ == "__main__":
     # Get training job name
     print(f"Training Job: {my_model.training_job_name}")
 
+    # List any inference runs
+    print(f"Inference Runs: {my_model.list_inference_runs()}")
+
     # Get any captured metrics from the training job
-    print("Training Metrics:")
-    print(my_model.model_metrics())
+    print("Model Metrics:")
+    print(my_model.performance_metrics())
 
     print("Confusion Matrix: (might be None)")
     print(my_model.confusion_matrix())
 
     # Grab our regression predictions from S3
-    print("Regression Predictions: (might be None)")
-    print(my_model.regression_predictions())
+    print("Captured Predictions: (might be None)")
+    print(my_model.predictions())
 
     # Grab our Shapley values from S3
     print("Shapley Values: (might be None)")
-    print(my_model.model_shapley_values())
+    print(my_model.shapley_values())
 
     # Get the SageWorks metadata associated with this Model
     print(f"SageWorks Meta: {my_model.sageworks_meta()}")
@@ -582,5 +824,11 @@ if __name__ == "__main__":
     sagemaker_model = my_model.latest_model_object()
     print(f"Latest Model Object: {my_model.latest_model_object()}")
 
+    # Get the Class Labels (if it's a classifier)
+    my_model = ModelCore("wine-classification")
+    print(f"Class Labels: {my_model.class_labels()}")
+    my_model.set_class_labels(["red", "white"])
+    print(f"Class Labels: {my_model.class_labels()}")
+
     # Delete the Model
     # my_model.delete()
