@@ -1,4 +1,5 @@
 """FeatureSetWebView pulls FeatureSet metadata from the AWS Service Broker with Details Panels on each FeatureSet"""
+
 import pandas as pd
 
 # SageWorks Imports
@@ -17,7 +18,6 @@ class FeatureSetWebView(ArtifactsWebView):
 
     def refresh(self):
         """Refresh the data from the AWS Service Broker"""
-        super().refresh()
         self.feature_sets_df = self.feature_sets_summary()
 
     def view_data(self) -> pd.DataFrame:
@@ -89,5 +89,9 @@ if __name__ == "__main__":
     print(sample_df.shape)
     print(sample_df.head())
 
+    # Test refresh
+    print("\nRefreshing...")
+    feature_view.refresh()
+
     # Give any broker threads time to finish
     time.sleep(1)
