@@ -1,4 +1,5 @@
 """ModelWebView pulls Model metadata from the AWS Service Broker with Details Panels on each Model"""
+
 import pandas as pd
 
 # SageWorks Imports
@@ -17,7 +18,6 @@ class ModelWebView(ArtifactsWebView):
 
     def refresh(self):
         """Refresh the data from the AWS Service Broker"""
-        super().refresh()
         self.models_df = self.models_summary()
 
     def view_data(self) -> pd.DataFrame:
@@ -39,7 +39,7 @@ class ModelWebView(ArtifactsWebView):
         if not model.exists():
             return {"Status": "Not Found"}
         elif not model.ready():
-            return {"health_tags": model.sageworks_health_tags()}
+            return {"health_tags": model.get_health_tags()}
 
         # Return the Model Details
         return model.details()
