@@ -1,4 +1,5 @@
 """View: View in the database sense: Pulls from the AWS Service Broker and does slice and dice"""
+
 from abc import ABC, abstractmethod
 
 import logging
@@ -19,11 +20,6 @@ class View(ABC):
         self.boto_session = self.aws_account_clamp.boto_session()
         self.sm_session = self.aws_account_clamp.sagemaker_session()
 
-    @abstractmethod
-    def check(self) -> bool:
-        """Can we connect to this view/service?"""
-        pass
-
     @abstractmethod
     def refresh(self) -> bool:
         """Refresh data/metadata associated with this view"""
