@@ -1,30 +1,31 @@
 """An abstract class that defines the web component interface for SageWorks"""
+
 from abc import abstractmethod
 from inspect import signature
-import typing
+from typing import Union, get_args
 from enum import Enum
-from functools import wraps
 
 # Local Imports
 from sageworks.web_components.component_interface import ComponentInterface
 
 
-class PluginType(Enum):
-    """TBD: Nice Docstring here or link to docs"""
+class PluginPage(Enum):
+    """Plugin Page: Specify which page will autoload the plugin (CUSTOM = Don't autoload)"""
 
     DATA_SOURCE = "data_source"
     FEATURE_SET = "feature_set"
     MODEL = "model"
     ENDPOINT = "endpoint"
+    CUSTOM = "custom"
 
 
 class PluginInputType(Enum):
-    """TBD: Nice Docstring here or link to docs"""
+    """Plugin Input Type: Specify the type of object that the plugin will receive as input"""
 
-    DATA_SOURCE_DETAILS = "data_source_details"
-    FEATURE_SET_DETAILS = "feature_set_details"
-    MODEL_DETAILS = "model_details"
-    ENDPOINT_DETAILS = "endpoint_details"
+    DATA_SOURCE = "data_source"
+    FEATURE_SET = "feature_set"
+    MODEL = "model"
+    ENDPOINT = "endpoint"
 
 
 class PluginInterface(ComponentInterface):
@@ -33,7 +34,7 @@ class PluginInterface(ComponentInterface):
       - These methods are ^stateless^, all data should be passed through the
         arguments and the implementations should not reference 'self' variables
       - The 'create_component' method must be implemented by the child class
-      - The 'generate_component_figure' method must be implemented by the child class
+      - The 'update_contents' method must be implemented by the child class
     """
 
     @abstractmethod
@@ -47,10 +48,10 @@ class PluginInterface(ComponentInterface):
         pass
 
     @abstractmethod
-    def generate_component_figure(self, details: dict) -> ComponentInterface.FigureTypes:
+    def update_contents(self, data_object: ComponentInterface.SageworksObject) -> ComponentInterface.ContentTypes:
         """Generate a figure from the data in the given dataframe.
         Args:
-            details (dict): The details dictionary for the plugin type
+            data_object (sageworks_object): The instantiated data object for the plugin type.
         Returns:
             Union[go.Figure, str]: A Plotly Figure or a Markdown string
         """
@@ -62,9 +63,9 @@ class PluginInterface(ComponentInterface):
     def __init_subclass__(cls, **kwargs):
         super().__init_subclass__(**kwargs)
 
-        if not hasattr(cls, "plugin_type") or not isinstance(cls.plugin_type, PluginType):
-            raise TypeError("Subclasses must define a 'plugin_type' of type PluginType")
-
+        # Ensure the subclass defines the required plugin_page and plugin_input_type
+        if not hasattr(cls, "plugin_page") or not isinstance(cls.plugin_page, PluginPage):
+            raise TypeError("Subclasses must define a 'plugin_page' of type PluginPage")
         if not hasattr(cls, "plugin_input_type") or not isinstance(cls.plugin_input_type, PluginInputType):
             raise TypeError("Subclasses must define a 'plugin_input_type' of type PluginInputType")
 
@@ -75,7 +76,7 @@ class PluginInterface(ComponentInterface):
     def __subclasshook__(cls, subclass):
         if cls is PluginInterface:
             # Check if the subclass has all the required attributes
-            if not all(hasattr(subclass, attr) for attr in ("plugin_type", "plugin_input_type")):
+            if not all(hasattr(subclass, attr) for attr in ("plugin_page", "plugin_input_type")):
                 cls.log.warning(f"Subclass {subclass.__name__} is missing required attributes")
                 return False
 
@@ -87,14 +88,14 @@ class PluginInterface(ComponentInterface):
                     cls.log.warning(f"Subclass {subclass.__name__} is missing required method {method}")
                     return False
 
-                # Check if the method is different from the base class (i.e., it's been implemented)
-                base_class_method = getattr(cls, method)
+                # Check if the method is implemented by the subclass itself
                 subclass_method = getattr(subclass, method)
-                if subclass_method is getattr(PluginInterface, method):
+                if subclass_method.__qualname__.split(".")[0] != subclass.__name__:
                     cls.log.warning(f"Subclass {subclass.__name__} has not implemented the method {method}")
                     return False
 
                 # Check argument types
+                base_class_method = getattr(cls, method)
                 arg_type_error = cls._check_argument_types(base_class_method, subclass_method)
                 if arg_type_error:
                     cls.log.warning(f"Subclass {subclass.__name__} error in method '{method}': {arg_type_error}")
@@ -112,36 +113,32 @@ class PluginInterface(ComponentInterface):
 
     @classmethod
     def _check_argument_types(cls, base_class_method, subclass_method):
-        # Extract argument types, excluding 'self'
+        # Extract expected argument types, excluding 'self'
         expected_arg_types = [v for k, v in base_class_method.__annotations__.items() if k != "return" and k != "self"]
         actual_arg_types = [
             param.annotation for param in signature(subclass_method).parameters.values() if param.name != "self"
         ]
 
-        # Directly compare the lists of argument types
-        if expected_arg_types != actual_arg_types:
-            return f"Expected argument types {expected_arg_types} do not match actual argument types {actual_arg_types}"
+        # Iterate over expected and actual argument types together
+        for expected, actual in zip(expected_arg_types, actual_arg_types):
+            # If the expected type is a Union, use get_args to extract its arguments
+            if getattr(expected, "__origin__", None) is Union:
+                expected_types = get_args(expected)
+                # Check if the actual type is a subtype of any of the expected types
+                if not any(issubclass(actual, exp) for exp in expected_types):
+                    return f"Expected argument types {expected_types} do not include the actual argument type {actual}"
+            else:
+                # Direct comparison for non-Union types
+                if expected != actual:
+                    return f"Expected argument type {expected} does not match actual argument type {actual}"
         return None
 
     @classmethod
     def _check_return_type(cls, base_class_method, subclass_method):
         return_annotation = base_class_method.__annotations__["return"]
-        expected_return_types = typing.get_args(return_annotation)
+        expected_return_types = get_args(return_annotation)
         return_type = signature(subclass_method).return_annotation
 
         if return_type not in expected_return_types:
             return f"Incorrect return type (expected one of {expected_return_types}, got {return_type})"
         return None
-
-
-# This is a helper decorator to catch errors in plugin methods and return a message figure
-def plugin_error_decorator(func):
-    @wraps(func)
-    def wrapper(*args, **kwargs):
-        try:
-            return func(*args, **kwargs)
-        except Exception as e:
-            error_info = f"Plugin Crashed: {e.__class__.__name__}: {e}"
-            return ComponentInterface.message_figure(error_info)
-
-    return wrapper
