@@ -11,6 +11,7 @@ Models:
 Endpoints:
     - abalone-regression-end
 """
+
 import sys
 import logging
 from pathlib import Path
@@ -59,15 +60,32 @@ if __name__ == "__main__":
     # Create the abalone_regression Model
     if recreate or not Model("abalone-regression").exists():
         fs = FeatureSet("abalone_features")
-        fs.to_model(
+        m = fs.to_model(
             ModelType.REGRESSOR,
             name="abalone-regression",
             target_column="class_number_of_rings",
             tags=["abalone", "regression"],
             description="Abalone Regression Model",
         )
+        m.set_owner("test")
+
+    # Create the abalone_regression Model
+    if recreate or not Model("abalone-regression-full").exists():
+        fs = FeatureSet("abalone_features")
+        m = fs.to_model(
+            ModelType.REGRESSOR,
+            name="abalone-regression-full",
+            target_column="class_number_of_rings",
+            tags=["abalone", "regression"],
+            description="Abalone Regression Model",
+            train_all_data=True,
+        )
+        m.set_owner("test")
 
     # Create the abalone_regression Endpoint
     if recreate or not Endpoint("abalone-regression-end").exists():
         model = Model("abalone-regression")
-        model.to_endpoint(name="abalone-regression-end", tags=["abalone", "regression"])
+        end = model.to_endpoint(name="abalone-regression-end", tags=["abalone", "regression"])
+
+        # Run inference on the endpoint
+        end.auto_inference(capture=True)
