@@ -1,4 +1,5 @@
 """This Script Deletes the SageWorks Artifacts used for the tests"""
+
 import time
 from sageworks.api.data_source import DataSource
 from sageworks.api.feature_set import FeatureSet
@@ -11,61 +12,67 @@ if __name__ == "__main__":
     # This forces a refresh on all the data we get from the AWs Broker
     AWSServiceBroker().get_all_metadata(force_refresh=True)
 
-    # Delete the test_data DataSource
+    # Delete the test_data Artifacts
     ds = DataSource("test_data")
     if ds.exists():
         print("Deleting test_data...")
         ds.delete()
-
-    # Delete the abalone_data DataSource
-    ds = DataSource("abalone_data")
+    ds = DataSource("abc")
     if ds.exists():
-        print("Deleting abalone_data...")
+        print("Deleting abc...")
         ds.delete()
-
-    # Delete the abalone_data_copy DataSource
-    ds = DataSource("abalone_data_copy")
+    ds = DataSource("abc_2")
     if ds.exists():
-        print("Deleting abalone_data_copy...")
+        print("Deleting abc_2...")
         ds.delete()
-
-    # Delete the test_features FeatureSet
     fs = FeatureSet("test_features")
     if fs.exists():
         print("Deleting test_features...")
         fs.delete()
-
-    # Delete the abalone_features FeatureSet
-    fs = FeatureSet("abalone_features")
-    if fs.exists():
-        print("Deleting abalone_features...")
-        fs.delete()
-
-    # Delete the wine_feature FeatureSet
-    fs = FeatureSet("wine_features")
-    if fs.exists():
-        print("Deleting wine_features...")
-        fs.delete()
-
-    # Delete the test-model Model
     m = Model("test-model")
     if m.exists():
         print("Deleting test-model model...")
         m.delete()
+    end = Endpoint("test-end")
+    if end.exists():
+        print("Deleting test-end endpoint...")
+        end.delete()
+    end = Endpoint("abc")
+    if end.exists():
+        print("Deleting abc endpoint...")
+        end.delete()
+    end = Endpoint("abc-2")
+    if end.exists():
+        print("Deleting abc-2 endpoint...")
+        end.delete()
 
-    # Delete the abalone-regression Model
+    # Delete the abalone_data Artifacts
+    ds = DataSource("abalone_data")
+    if ds.exists():
+        print("Deleting abalone_data...")
+        ds.delete()
+    ds = DataSource("abalone_data_copy")
+    if ds.exists():
+        print("Deleting abalone_data_copy...")
+        ds.delete()
+    fs = FeatureSet("abalone_features")
+    if fs.exists():
+        print("Deleting abalone_features...")
+        fs.delete()
     m = Model("abalone-regression")
     if m.exists():
         print("Deleting abalone-regression model...")
         m.delete()
-
-    # Delete the abalone-regression Endpoint
     end = Endpoint("abalone-regression-end")
     if end.exists():
         print("Deleting abalone-regression-end...")
         end.delete()
+    end = Endpoint("abalone-regression-end-rt")
+    if end.exists():
+        print("Deleting abalone-regression-end-rt...")
+        end.delete()
 
-    # Classification Artifacts
+    # Delete abalone_classification Artifacts
     fs = FeatureSet("abalone_classification")
     if fs.exists():
         print("Deleting abalone_classification...")
@@ -79,7 +86,7 @@ if __name__ == "__main__":
         print("Deleting abalone-classification-end endpoint...")
         end.delete()
 
-    # Wine Artifacts
+    # Delete the wine_data Artifacts
     ds = DataSource("wine_data")
     if ds.exists():
         print("Deleting wine_data...")
@@ -114,17 +121,17 @@ if __name__ == "__main__":
     if end.exists():
         print("Deleting aqsol-regression-end endpoint...")
         end.delete()
-    fs = FeatureSet("aqsol_rdkit_features")
+    fs = FeatureSet("aqsol_mol_descriptors")
     if fs.exists():
-        print("Deleting aqsol_rdkit_features...")
+        print("Deleting aqsol_mol_descriptors...")
         fs.delete()
-    m = Model("aqsol-rdkit-regression")
+    m = Model("aqsol-descriptors-regression")
     if m.exists():
-        print("Deleting aqsol-rdkit-regression model...")
+        print("Deleting aqsol-mol-regression model...")
         m.delete()
-    end = Endpoint("aqsol-rdkit-regression-end")
+    end = Endpoint("aqsol-mol-regression-end")
     if end.exists():
-        print("Deleting aqsol-rdkit-regression-end endpoint...")
+        print("Deleting aqsol-mol-regression-end endpoint...")
         end.delete()
 
     time.sleep(5)
